# -*- cmake -*-
if (PYTHON_CMAKE_INCLUDED)
  return()
endif (PYTHON_CMAKE_INCLUDED)
set (PYTHON_CMAKE_INCLUDED TRUE)

set(PYTHONINTERP_FOUND)

if (WINDOWS)
	# On Windows, explicitly avoid Cygwin Python.
	foreach(hive HKEY_CURRENT_USER HKEY_LOCAL_MACHINE)
		# Note: 3.14 last since still not released (Oct-2024)
		foreach(pyver 3.13 3.12 3.11 3.10 3.9 3.8 3.7 3.9-32 3.8-32 3.7-32 3.6-32 3.5-32 3.4-32 3.3-32 3.14)
			list(APPEND regpaths "[${hive}\\SOFTWARE\\Python\\PythonCore\\${pyver}\\InstallPath]")
		endforeach()
	endforeach()

	find_program(PYTHON_EXECUTABLE
		NAMES python3.exe python.exe
		NO_DEFAULT_PATH # added so that cmake does not find cygwin python
		PATHS
		${regpaths}
    )
elseif (LINUX)
	string(REPLACE ":" ";" PATH_LIST "$ENV{PATH}")
	# Note: 3.14 last since still not released (Oct-2024)
	find_program(PYTHON_EXECUTABLE python3.13 python3.12 python3.11 python3.10 python3.9 python3.8 python3.7 python3.6 python3.5 python3.4 python3.3 python3 python3.14 python
				 PATHS ${PATH_LIST})
elseif (DARWIN)
	# On MAC OS X be sure to search standard locations first
	string(REPLACE ":" ";" PATH_LIST "$ENV{PATH}")
	# Note: 3.14 last since still not released (Oct-2024)
	find_program(PYTHON_EXECUTABLE
    NAMES python313 python312 python311 python310 python39 python38 python37 python36 python35 python34 python33 python3 python314 python
    NO_DEFAULT_PATH # Avoid searching non-standard locations first
    PATHS
    /bin
    /usr/bin
    /usr/local/bin
    ${PATH_LIST}
    )
endif (WINDOWS)

if (PYTHON_EXECUTABLE)
	set(PYTHONINTERP_FOUND ON)
endif (PYTHON_EXECUTABLE)

if (NOT PYTHONINTERP_FOUND)
  message(FATAL_ERROR "No compatible Python interpreter found !")
endif (NOT PYTHONINTERP_FOUND)

mark_as_advanced(PYTHON_EXECUTABLE)
message("-- Using Python executable: ${PYTHON_EXECUTABLE}")
