/**
 * @file llsdjson.h
 * @brief LLSD flexible data system
 *
 * $LicenseInfo:firstyear=2015&license=viewergpl$
 *
 * Copyright (c) 2015, Linden Research, Inc.
 *
 * Second Life Viewer Source Code
 * The source code in this file ("Source Code") is provided by Linden Lab
 * to you under the terms of the GNU General Public License, version 2.0
 * ("GPL"), unless you have obtained a separate licensing agreement
 * ("Other License"), formally executed by you and Linden Lab.  Terms of
 * the GPL can be found in doc/GPL-license.txt in this distribution, or
 * online at http://secondlifegrid.net/programs/open_source/licensing/gplv2
 *
 * There are special exceptions to the terms and conditions of the GPL as
 * it is applied to this Source Code. View the full text of the exception
 * in the file doc/FLOSS-exception.txt in this software distribution, or
 * online at
 * http://secondlifegrid.net/programs/open_source/licensing/flossexception
 *
 * By copying, modifying or distributing this software, you acknowledge
 * that you have read and understood your obligations described above,
 * and agree to abide by those obligations.
 *
 * ALL LINDEN LAB SOURCE CODE IS PROVIDED "AS IS." LINDEN LAB MAKES NO
 * WARRANTIES, EXPRESS, IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY,
 * COMPLETENESS OR PERFORMANCE.
 * $/LicenseInfo$
 */

#ifndef LL_LLSDJSON_H
#define LL_LLSDJSON_H

#include "json.hpp"

using lljson = nlohmann::json;

#include "llsd.h"

// Convert a parsed JSON structure into LLSD maintaining member names and
// array indexes.
// JSON/JavaScript types are converted as follows:
//
// JSON Type     | LLSD Type
// --------------+--------------
//  null         |  undefined
//  integer      |  LLSD::Integer
//  unsigned     |  LLSD::Integer
//  real/numeric |  LLSD::Real
//  string       |  LLSD::String
//  boolean      |  LLSD::Boolean
//  array        |  LLSD::Array
//  object       |  LLSD::Map
//
// For maps and arrays child entries will be converted and added to the structure.
// Order is preserved for an array but not for objects.
LLSD llsd_from_json(const lljson& val);

// Convert an LLSD object into Parsed JSON object maintaining member names and
// array indexs.
//
// Types are converted as follows:
// LLSD Type     |  JSON Type
// --------------+----------------
// TypeUndefined | null
// TypeBoolean   | boolean
// TypeInteger   | integer
// TypeReal      | real/numeric
// TypeString    | string
// TypeURI       | string
// TypeDate      | string
// TypeUUID      | string
// TypeMap       | object
// TypeArray     | array
// TypeBinary    | unsupported
lljson llsd_to_json(const LLSD& val);

#endif	// LL_LLSDJSON_H
