/**
 * @file llstrider.h
 *
 * $LicenseInfo:firstyear=2001&license=viewergpl$
 *
 * Copyright (c) 2001-2009, Linden Research, Inc.
 *
 * Second Life Viewer Source Code
 * The source code in this file ("Source Code") is provided by Linden Lab
 * to you under the terms of the GNU General Public License, version 2.0
 * ("GPL"), unless you have obtained a separate licensing agreement
 * ("Other License"), formally executed by you and Linden Lab.  Terms of
 * the GPL can be found in doc/GPL-license.txt in this distribution, or
 * online at http://secondlifegrid.net/programs/open_source/licensing/gplv2
 *
 * There are special exceptions to the terms and conditions of the GPL as
 * it is applied to this Source Code. View the full text of the exception
 * in the file doc/FLOSS-exception.txt in this software distribution, or
 * online at
 * http://secondlifegrid.net/programs/open_source/licensing/flossexception
 *
 * By copying, modifying or distributing this software, you acknowledge
 * that you have read and understood your obligations described above,
 * and agree to abide by those obligations.
 *
 * ALL LINDEN LAB SOURCE CODE IS PROVIDED "AS IS." LINDEN LAB MAKES NO
 * WARRANTIES, EXPRESS, IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY,
 * COMPLETENESS OR PERFORMANCE.
 * $/LicenseInfo$
 */

#ifndef LL_LLSTRIDER_H
#define LL_LLSTRIDER_H

#include "stdtypes.h"

#include "llpreprocessor.h"

template <class Object> class LLStrider
{
public:
	LL_INLINE LLStrider()
	:	mObjectp(NULL),
		mSkip(sizeof(Object))
	{
	}

	LL_INLINE LLStrider(Object* firstp)
	:	mObjectp(firstp),
		mSkip(sizeof(Object))
	{
	}

	~LLStrider() = default;

	LL_INLINE const LLStrider<Object>& operator=(const LLStrider<Object>& rhs)
	{
		mBytep = rhs.mBytep;
		mSkip = rhs.mSkip;
		return *this;
	}

	LL_INLINE const LLStrider<Object>& operator=(Object* firstp)
	{
		mObjectp = firstp;
		return *this;
	}

	LL_INLINE void setStride(S32 skip)
	{
		mSkip = skip ? skip : sizeof(Object);
	}

	LL_INLINE LLStrider<Object> operator+(const S32& index)
	{
		LLStrider<Object> ret;
		ret.mBytep = mBytep + mSkip * index;
		ret.mSkip = mSkip;
		return ret;
	}

	LL_INLINE void skip(U32 index)				{ mBytep += mSkip * index;}
	LL_INLINE U32 getSkip() const				{ return mSkip; }
	LL_INLINE Object* get()						{ return mObjectp; }
	LL_INLINE const Object* get() const			{ return mObjectp; }
	LL_INLINE Object* operator->()				{ return mObjectp; }
	LL_INLINE Object& operator*()				{ return *mObjectp; }

	LL_INLINE Object* operator++(int)
	{
		Object* old = mObjectp;
		mBytep += mSkip;
		return old;
	}

	LL_INLINE Object* operator+=(int i)
	{
		mBytep += mSkip * i;
		return mObjectp;
	}

	LL_INLINE Object& operator[](U32 index)
	{
		return *(Object*)(mBytep + mSkip * index);
	}

private:
	union
	{
		Object*	mObjectp;
		U8*		mBytep;
	};

	U32			mSkip;
};

#endif // LL_LLSTRIDER_H
