/**
 * @file llgltfglm.h
 * @brief LLGLTF GLM math support
 *
 * $LicenseInfo:firstyear=2024&license=viewergpl$
 *
 * Copyright (c) 2024, Linden Research, Inc & Henri Beauchamp.
 *
 * Second Life Viewer Source Code
 * The source code in this file ("Source Code") is provided by Linden Lab
 * to you under the terms of the GNU General Public License, version 2.0
 * ("GPL"), unless you have obtained a separate licensing agreement
 * ("Other License"), formally executed by you and Linden Lab.  Terms of
 * the GPL can be found in doc/GPL-license.txt in this distribution, or
 * online at http://secondlifegrid.net/programs/open_source/licensing/gplv2
 *
 * There are special exceptions to the terms and conditions of the GPL as
 * it is applied to this Source Code. View the full text of the exception
 * in the file doc/FLOSS-exception.txt in this software distribution, or
 * online at
 * http://secondlifegrid.net/programs/open_source/licensing/flossexception
 *
 * By copying, modifying or distributing this software, you acknowledge
 * that you have read and understood your obligations described above,
 * and agree to abide by those obligations.
 *
 * ALL LINDEN LAB SOURCE CODE IS PROVIDED "AS IS." LINDEN LAB MAKES NO
 * WARRANTIES, EXPRESS, IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY,
 * COMPLETENESS OR PERFORMANCE.
 * $/LicenseInfo$
 */

#ifndef LL_LLGLTFGLM_H
#define LL_LLGLTFGLM_H

// Make sure the proper intrinsics header is included. HB
#include "hbintrinsics.h"

// Verify things were properly set up for GLM... This block allows to skip
// include(GLM) for newview, as long as all modules using GLM in the latter
// do include this header (which is the case whenever llgltfasset.h is itself
// included, and since newview only cares about assets, it is all good). HB
#if !GLM_FORCE_DEFAULT_ALIGNED_GENTYPES
# define GLM_FORCE_DEFAULT_ALIGNED_GENTYPES 1
# define GLM_ENABLE_EXPERIMENTAL 1
// GLM does not have proper NEON support, so we use sse2neon.h translations
// (done via hbintrinsics.h) and we force SSE2 intrinsics usage by GLM. HB
# if SSE2NEON
#  define GLM_FORCE_ARCH_UNKNOWN 1
#  define GLM_FORCE_SSE2 1
# endif
// Enable inlining optimizations, but only for release builds. HB
# if !LL_DEBUG && !LL_NO_FORCE_INLINE
#  define GLM_FORCE_INLINE 1
# endif
#endif
// Include the headers for the GLM types and features we do use.
#include "glm/vec2.hpp"
#include "glm/vec3.hpp"
#include "glm/vec4.hpp"
#include "glm/mat4x4.hpp"
#include "glm/gtc/type_ptr.hpp"
#include "glm/ext/quaternion_float.hpp"
#include "glm/gtx/quaternion.hpp"
#include "glm/gtx/matrix_decompose.hpp"

// Common types and constants used in the LLGLTF implementation
namespace LLGLTF
{
	using mat4 = glm::mat4;
	using vec4 = glm::vec4;
	using vec3 = glm::vec3;
	using vec2 = glm::vec2;
	using quat = glm::quat;

	// Used by LLGLTFSceneManager for rendering.
	enum TextureType : U8
	{
		BASE_COLOR = 0,
		NORMAL,
		METALLIC_ROUGHNESS,
		OCCLUSION,
		EMISSIVE,
		TEXTURE_TYPE_COUNT
	};
}

#endif	// LL_LLGLTFGLM_H
