/**
 * @file message_prehash.cpp
 * @brief file of prehashed variables
 *
 * $LicenseInfo:firstyear=2003&license=viewergpl$
 *
 * Copyright (c) 2003-2009, Linden Research, Inc.
 *
 * Second Life Viewer Source Code
 * The source code in this file ("Source Code") is provided by Linden Lab
 * to you under the terms of the GNU General Public License, version 2.0
 * ("GPL"), unless you have obtained a separate licensing agreement
 * ("Other License"), formally executed by you and Linden Lab.  Terms of
 * the GPL can be found in doc/GPL-license.txt in this distribution, or
 * online at http://secondlifegrid.net/programs/open_source/licensing/gplv2
 *
 * There are special exceptions to the terms and conditions of the GPL as
 * it is applied to this Source Code. View the full text of the exception
 * in the file doc/FLOSS-exception.txt in this software distribution, or
 * online at
 * http://secondlifegrid.net/programs/open_source/licensing/flossexception
 *
 * By copying, modifying or distributing this software, you acknowledge
 * that you have read and understood your obligations described above,
 * and agree to abide by those obligations.
 *
 * ALL LINDEN LAB SOURCE CODE IS PROVIDED "AS IS." LINDEN LAB MAKES NO
 * WARRANTIES, EXPRESS, IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY,
 * COMPLETENESS OR PERFORMANCE.
 * $/LicenseInfo$
 */

#include "linden_common.h"

#include "llmessage.h"

// We initialize this global variable here (and not in llmessage.cpp for
// example), so to be sure that, whatever how the linker worked it out,
// gMessageStringTable is valid when we initialize the constant variables
// below. This also ensures that gMessageStringTable is initialized at the
// earliest stage and ready to be used by other classes when they get
// instantiated in their turn. HB
LLMessageStringTable gMessageStringTable;

F32 const gPrehashVersionNumber = 2.000f;

// The strings must match the ones used in the current message_template.msg
// file
char const* const _PREHASH_X = gMessageStringTable.getString("X");
char const* const _PREHASH_Y = gMessageStringTable.getString("Y");
char const* const _PREHASH_Z = gMessageStringTable.getString("Z");
char const* const _PREHASH_AddFlags = gMessageStringTable.getString("AddFlags");
char const* const _PREHASH_FailureInfo = gMessageStringTable.getString("FailureInfo");
char const* const _PREHASH_MapData = gMessageStringTable.getString("MapData");
char const* const _PREHASH_AddItem = gMessageStringTable.getString("AddItem");
char const* const _PREHASH_MeanCollision = gMessageStringTable.getString("MeanCollision");
char const* const _PREHASH_RezScript = gMessageStringTable.getString("RezScript");
char const* const _PREHASH_AvatarSitResponse = gMessageStringTable.getString("AvatarSitResponse");
char const* const _PREHASH_InventoryAssetResponse = gMessageStringTable.getString("InventoryAssetResponse");
char const* const _PREHASH_KillObject = gMessageStringTable.getString("KillObject");
char const* const _PREHASH_ProposalID = gMessageStringTable.getString("ProposalID");
char const* const _PREHASH_SerialNum = gMessageStringTable.getString("SerialNum");
char const* const _PREHASH_Duration = gMessageStringTable.getString("Duration");
char const* const _PREHASH_ScriptQuestion = gMessageStringTable.getString("ScriptQuestion");
char const* const _PREHASH_AddCircuitCode = gMessageStringTable.getString("AddCircuitCode");
char const* const _PREHASH_UseCircuitCode = gMessageStringTable.getString("UseCircuitCode");
char const* const _PREHASH_ViewerCircuitCode = gMessageStringTable.getString("ViewerCircuitCode");
char const* const _PREHASH_ScriptAnswerYes = gMessageStringTable.getString("ScriptAnswerYes");
char const* const _PREHASH_PartnerID = gMessageStringTable.getString("PartnerID");
char const* const _PREHASH_DirLandQuery = gMessageStringTable.getString("DirLandQuery");
char const* const _PREHASH_TeleportStart = gMessageStringTable.getString("TeleportStart");
char const* const _PREHASH_AboutText = gMessageStringTable.getString("AboutText");
char const* const _PREHASH_VisualParam = gMessageStringTable.getString("VisualParam");
char const* const _PREHASH_GroupPrims = gMessageStringTable.getString("GroupPrims");
char const* const _PREHASH_SelectedPrims = gMessageStringTable.getString("SelectedPrims");
char const* const _PREHASH_ID = gMessageStringTable.getString("ID");
char const* const _PREHASH_UUIDNameRequest = gMessageStringTable.getString("UUIDNameRequest");
char const* const _PREHASH_UUIDGroupNameRequest = gMessageStringTable.getString("UUIDGroupNameRequest");
char const* const _PREHASH_GroupAccountTransactionsRequest = gMessageStringTable.getString("GroupAccountTransactionsRequest");
char const* const _PREHASH_MapNameRequest = gMessageStringTable.getString("MapNameRequest");
char const* const _PREHASH_UpdateSimulator = gMessageStringTable.getString("UpdateSimulator");
char const* const _PREHASH_BillableFactor = gMessageStringTable.getString("BillableFactor");
char const* const _PREHASH_ObjectBonusFactor = gMessageStringTable.getString("ObjectBonusFactor");
char const* const _PREHASH_EnableSimulator = gMessageStringTable.getString("EnableSimulator");
char const* const _PREHASH_DisableSimulator = gMessageStringTable.getString("DisableSimulator");
char const* const _PREHASH_ConfirmEnableSimulator = gMessageStringTable.getString("ConfirmEnableSimulator");
char const* const _PREHASH_LayerType = gMessageStringTable.getString("LayerType");
char const* const _PREHASH_OwnerRole = gMessageStringTable.getString("OwnerRole");
char const* const _PREHASH_ParcelOverlay = gMessageStringTable.getString("ParcelOverlay");
char const* const _PREHASH_GroupOwned = gMessageStringTable.getString("GroupOwned");
char const* const _PREHASH_IP = gMessageStringTable.getString("IP");
char const* const _PREHASH_ChatFromViewer = gMessageStringTable.getString("ChatFromViewer");
char const* const _PREHASH_AvgAgentsInView = gMessageStringTable.getString("AvgAgentsInView");
char const* const _PREHASH_AgentsInView = gMessageStringTable.getString("AgentsInView");
char const* const _PREHASH_GroupTitle = gMessageStringTable.getString("GroupTitle");
char const* const _PREHASH_MapLayerReply = gMessageStringTable.getString("MapLayerReply");
char const* const _PREHASH_CompoundMsgID = gMessageStringTable.getString("CompoundMsgID");
char const* const _PREHASH_CameraConstraint = gMessageStringTable.getString("CameraConstraint");
char const* const _PREHASH_DownloadTotals = gMessageStringTable.getString("DownloadTotals");
char const* const _PREHASH_GenCounter = gMessageStringTable.getString("GenCounter");
char const* const _PREHASH_FrozenData = gMessageStringTable.getString("FrozenData");
char const* const _PREHASH_ChildAgentDying = gMessageStringTable.getString("ChildAgentDying");
char const* const _PREHASH_To = gMessageStringTable.getString("To");
char const* const _PREHASH_CopyInventoryFromNotecard = gMessageStringTable.getString("CopyInventoryFromNotecard");
char const* const _PREHASH_RezObjectFromNotecard = gMessageStringTable.getString("RezObjectFromNotecard");
char const* const _PREHASH_ParcelDirFeeCurrent = gMessageStringTable.getString("ParcelDirFeeCurrent");
char const* const _PREHASH_SeedCapability = gMessageStringTable.getString("SeedCapability");
char const* const _PREHASH_ObjectDuplicate = gMessageStringTable.getString("ObjectDuplicate");
char const* const _PREHASH_InventoryData = gMessageStringTable.getString("InventoryData");
char const* const _PREHASH_ReplyData = gMessageStringTable.getString("ReplyData");
char const* const _PREHASH_ResetList = gMessageStringTable.getString("ResetList");
char const* const _PREHASH_MediaID = gMessageStringTable.getString("MediaID");
char const* const _PREHASH_MediaAllowNavigate = gMessageStringTable.getString("MediaAllowNavigate");
char const* const _PREHASH_MediaAutoScale = gMessageStringTable.getString("MediaAutoScale");
char const* const _PREHASH_MediaCurrentURL = gMessageStringTable.getString("MediaCurrentURL");
char const* const _PREHASH_MediaData = gMessageStringTable.getString("MediaData");
char const* const _PREHASH_MediaDesc = gMessageStringTable.getString("MediaDesc");
char const* const _PREHASH_MediaHeight = gMessageStringTable.getString("MediaHeight");
char const* const _PREHASH_MediaLinkSharing = gMessageStringTable.getString("MediaLinkSharing");
char const* const _PREHASH_MediaLoop = gMessageStringTable.getString("MediaLoop");
char const* const _PREHASH_MediaPreventCameraZoom = gMessageStringTable.getString("MediaPreventCameraZoom");
char const* const _PREHASH_MediaURLTimeout = gMessageStringTable.getString("MediaURLTimeout");
char const* const _PREHASH_MediaType = gMessageStringTable.getString("MediaType");
char const* const _PREHASH_MediaWidth = gMessageStringTable.getString("MediaWidth");
char const* const _PREHASH_RelatedRights = gMessageStringTable.getString("RelatedRights");
char const* const _PREHASH_RedirectGridX = gMessageStringTable.getString("RedirectGridX");
char const* const _PREHASH_RedirectGridY = gMessageStringTable.getString("RedirectGridY");
char const* const _PREHASH_TransferID = gMessageStringTable.getString("TransferID");
char const* const _PREHASH_TexturesChanged = gMessageStringTable.getString("TexturesChanged");
char const* const _PREHASH_UserLookAt = gMessageStringTable.getString("UserLookAt");
char const* const _PREHASH_TestBlock1 = gMessageStringTable.getString("TestBlock1");
char const* const _PREHASH_SensedData = gMessageStringTable.getString("SensedData");
char const* const _PREHASH_UpdateBlock = gMessageStringTable.getString("UpdateBlock");
char const* const _PREHASH_ClassifiedGodDelete = gMessageStringTable.getString("ClassifiedGodDelete");
char const* const _PREHASH_ObjectGrabUpdate = gMessageStringTable.getString("ObjectGrabUpdate");
char const* const _PREHASH_LocationPos = gMessageStringTable.getString("LocationPos");
char const* const _PREHASH_TaxDate = gMessageStringTable.getString("TaxDate");
char const* const _PREHASH_StartDateTime = gMessageStringTable.getString("StartDateTime");
char const* const _PREHASH_ObjectUpdateCached = gMessageStringTable.getString("ObjectUpdateCached");
char const* const _PREHASH_Packets = gMessageStringTable.getString("Packets");
char const* const _PREHASH_FailureType = gMessageStringTable.getString("FailureType");
char const* const _PREHASH_UpdateGroupInfo = gMessageStringTable.getString("UpdateGroupInfo");
char const* const _PREHASH_ObjectPermissions = gMessageStringTable.getString("ObjectPermissions");
char const* const _PREHASH_RevokePermissions = gMessageStringTable.getString("RevokePermissions");
char const* const _PREHASH_UpdateFlags = gMessageStringTable.getString("UpdateFlags");
char const* const _PREHASH_ObjectExportSelected = gMessageStringTable.getString("ObjectExportSelected");
char const* const _PREHASH_RezSelected = gMessageStringTable.getString("RezSelected");
char const* const _PREHASH_AutoPilot = gMessageStringTable.getString("AutoPilot");
char const* const _PREHASH_UpdateMuteListEntry = gMessageStringTable.getString("UpdateMuteListEntry");
char const* const _PREHASH_RemoveMuteListEntry = gMessageStringTable.getString("RemoveMuteListEntry");
char const* const _PREHASH_SetSimStatusInDatabase = gMessageStringTable.getString("SetSimStatusInDatabase");
char const* const _PREHASH_SetSimPresenceInDatabase = gMessageStringTable.getString("SetSimPresenceInDatabase");
char const* const _PREHASH_CameraProperty = gMessageStringTable.getString("CameraProperty");
char const* const _PREHASH_BrushSize = gMessageStringTable.getString("BrushSize");
char const* const _PREHASH_SimulatorSetMap = gMessageStringTable.getString("SimulatorSetMap");
char const* const _PREHASH_RegionPresenceRequestByRegionID = gMessageStringTable.getString("RegionPresenceRequestByRegionID");
char const* const _PREHASH_ParcelObjectOwnersReply = gMessageStringTable.getString("ParcelObjectOwnersReply");
char const* const _PREHASH_GroupMembersReply = gMessageStringTable.getString("GroupMembersReply");
char const* const _PREHASH_GroupRoleMembersReply = gMessageStringTable.getString("GroupRoleMembersReply");
char const* const _PREHASH_RequestRegionInfo = gMessageStringTable.getString("RequestRegionInfo");
char const* const _PREHASH_AABBMax = gMessageStringTable.getString("AABBMax");
char const* const _PREHASH_RequestPayPrice = gMessageStringTable.getString("RequestPayPrice");
char const* const _PREHASH_SimulatorPresentAtLocation = gMessageStringTable.getString("SimulatorPresentAtLocation");
char const* const _PREHASH_AgentRequestSit = gMessageStringTable.getString("AgentRequestSit");
char const* const _PREHASH_AABBMin = gMessageStringTable.getString("AABBMin");
char const* const _PREHASH_ClassifiedFlags = gMessageStringTable.getString("ClassifiedFlags");
char const* const _PREHASH_ControlFlags = gMessageStringTable.getString("ControlFlags");
char const* const _PREHASH_TeleportRequest = gMessageStringTable.getString("TeleportRequest");
char const* const _PREHASH_ScriptTeleportRequest = gMessageStringTable.getString("ScriptTeleportRequest");
char const* const _PREHASH_EstateCovenantRequest = gMessageStringTable.getString("EstateCovenantRequest");
char const* const _PREHASH_DateUTC = gMessageStringTable.getString("DateUTC");
char const* const _PREHASH_TaskIDs = gMessageStringTable.getString("TaskIDs");
char const* const _PREHASH_RequestResult = gMessageStringTable.getString("RequestResult");
char const* const _PREHASH_CanAcceptAgents = gMessageStringTable.getString("CanAcceptAgents");
char const* const _PREHASH_ObjectSaleInfo = gMessageStringTable.getString("ObjectSaleInfo");
char const* const _PREHASH_KillChildAgents = gMessageStringTable.getString("KillChildAgents");
char const* const _PREHASH_Balance = gMessageStringTable.getString("Balance");
char const* const _PREHASH_DerezContainer = gMessageStringTable.getString("DerezContainer");
char const* const _PREHASH_ObjectData = gMessageStringTable.getString("ObjectData");
char const* const _PREHASH_CameraAtAxis = gMessageStringTable.getString("CameraAtAxis");
char const* const _PREHASH_InfoBlock = gMessageStringTable.getString("InfoBlock");
char const* const _PREHASH_OwnershipCost = gMessageStringTable.getString("OwnershipCost");
char const* const _PREHASH_AvatarNotesUpdate = gMessageStringTable.getString("AvatarNotesUpdate");
char const* const _PREHASH_PID = gMessageStringTable.getString("PID");
char const* const _PREHASH_DirPopularReply = gMessageStringTable.getString("DirPopularReply");
char const* const _PREHASH_SimData = gMessageStringTable.getString("SimData");
char const* const _PREHASH_TerrainHeightRange00 = gMessageStringTable.getString("TerrainHeightRange00");
char const* const _PREHASH_TerrainHeightRange01 = gMessageStringTable.getString("TerrainHeightRange01");
char const* const _PREHASH_TerrainHeightRange10 = gMessageStringTable.getString("TerrainHeightRange10");
char const* const _PREHASH_TerrainHeightRange11 = gMessageStringTable.getString("TerrainHeightRange11");
char const* const _PREHASH_UpdateInventoryItem = gMessageStringTable.getString("UpdateInventoryItem");
char const* const _PREHASH_UpdateCreateInventoryItem = gMessageStringTable.getString("UpdateCreateInventoryItem");
char const* const _PREHASH_MoveInventoryItem = gMessageStringTable.getString("MoveInventoryItem");
char const* const _PREHASH_CopyInventoryItem = gMessageStringTable.getString("CopyInventoryItem");
char const* const _PREHASH_LinkInventoryItem = gMessageStringTable.getString("LinkInventoryItem");
char const* const _PREHASH_RemoveInventoryItem = gMessageStringTable.getString("RemoveInventoryItem");
char const* const _PREHASH_CreateInventoryItem = gMessageStringTable.getString("CreateInventoryItem");
char const* const _PREHASH_PathTwistBegin = gMessageStringTable.getString("PathTwistBegin");
char const* const _PREHASH_CRC = gMessageStringTable.getString("CRC");
char const* const _PREHASH_AttachmentPoint = gMessageStringTable.getString("AttachmentPoint");
char const* const _PREHASH_TelehubBlock = gMessageStringTable.getString("TelehubBlock");
char const* const _PREHASH_FOVBlock = gMessageStringTable.getString("FOVBlock");
char const* const _PREHASH_StartLocationData = gMessageStringTable.getString("StartLocationData");
char const* const _PREHASH_PositionData = gMessageStringTable.getString("PositionData");
char const* const _PREHASH_TimeSinceLast = gMessageStringTable.getString("TimeSinceLast");
char const* const _PREHASH_MapImage = gMessageStringTable.getString("MapImage");
char const* const _PREHASH_Objects = gMessageStringTable.getString("Objects");
char const* const _PREHASH_URL = gMessageStringTable.getString("URL");
char const* const _PREHASH_CreationDate = gMessageStringTable.getString("CreationDate");
char const* const _PREHASH_JointPivot = gMessageStringTable.getString("JointPivot");
char const* const _PREHASH_FPS = gMessageStringTable.getString("FPS");
char const* const _PREHASH_HasTelehub = gMessageStringTable.getString("HasTelehub");
char const* const _PREHASH_PathEnd = gMessageStringTable.getString("PathEnd");
char const* const _PREHASH_ScriptDataReply = gMessageStringTable.getString("ScriptDataReply");
char const* const _PREHASH_MapBlockReply = gMessageStringTable.getString("MapBlockReply");
char const* const _PREHASH_PropertiesData = gMessageStringTable.getString("PropertiesData");
char const* const _PREHASH_ViewerEffect = gMessageStringTable.getString("ViewerEffect");
char const* const _PREHASH_FreezeUser = gMessageStringTable.getString("FreezeUser");
char const* const _PREHASH_OwnerPrims = gMessageStringTable.getString("OwnerPrims");
char const* const _PREHASH_ObjectGrab = gMessageStringTable.getString("ObjectGrab");
char const* const _PREHASH_ToAgentID = gMessageStringTable.getString("ToAgentID");
char const* const _PREHASH_SimulatorMapUpdate = gMessageStringTable.getString("SimulatorMapUpdate");
char const* const _PREHASH_TransferPacket = gMessageStringTable.getString("TransferPacket");
char const* const _PREHASH_ObjectName = gMessageStringTable.getString("ObjectName");
char const* const _PREHASH_GroupPowers = gMessageStringTable.getString("GroupPowers");
char const* const _PREHASH_OriginalName = gMessageStringTable.getString("OriginalName");
char const* const _PREHASH_CompletePingCheck = gMessageStringTable.getString("CompletePingCheck");
char const* const _PREHASH_OnlineStatus = gMessageStringTable.getString("OnlineStatus");
char const* const _PREHASH_ObjectDrop = gMessageStringTable.getString("ObjectDrop");
char const* const _PREHASH_UseBigPackets = gMessageStringTable.getString("UseBigPackets");
char const* const _PREHASH_GroupNoticesListReply = gMessageStringTable.getString("GroupNoticesListReply");
char const* const _PREHASH_ParcelAccessListReply = gMessageStringTable.getString("ParcelAccessListReply");
char const* const _PREHASH_RpcChannelReply = gMessageStringTable.getString("RpcChannelReply");
char const* const _PREHASH_RegionPresenceResponse = gMessageStringTable.getString("RegionPresenceResponse");
char const* const _PREHASH_CharterMember = gMessageStringTable.getString("CharterMember");
char const* const _PREHASH_EdgeData = gMessageStringTable.getString("EdgeData");
char const* const _PREHASH_NameData = gMessageStringTable.getString("NameData");
char const* const _PREHASH_RegionPushOverride = gMessageStringTable.getString("RegionPushOverride");
char const* const _PREHASH_SimName = gMessageStringTable.getString("SimName");
char const* const _PREHASH_UserReport = gMessageStringTable.getString("UserReport");
char const* const _PREHASH_DownloadPriority = gMessageStringTable.getString("DownloadPriority");
char const* const _PREHASH_ToAgentId = gMessageStringTable.getString("ToAgentId");
char const* const _PREHASH_DirPopularQuery = gMessageStringTable.getString("DirPopularQuery");
char const* const _PREHASH_Mag = gMessageStringTable.getString("Mag");
char const* const _PREHASH_ParcelPropertiesRequestByID = gMessageStringTable.getString("ParcelPropertiesRequestByID");
char const* const _PREHASH_ObjectLink = gMessageStringTable.getString("ObjectLink");
char const* const _PREHASH_RpcScriptReplyInbound = gMessageStringTable.getString("RpcScriptReplyInbound");
char const* const _PREHASH_RezData = gMessageStringTable.getString("RezData");
char const* const _PREHASH_RemoveInventoryObjects = gMessageStringTable.getString("RemoveInventoryObjects");
char const* const _PREHASH_GroupProposalBallot = gMessageStringTable.getString("GroupProposalBallot");
char const* const _PREHASH_RPCServerIP = gMessageStringTable.getString("RPCServerIP");
char const* const _PREHASH_Far = gMessageStringTable.getString("Far");
char const* const _PREHASH_GodSessionID = gMessageStringTable.getString("GodSessionID");
char const* const _PREHASH_FLAboutText = gMessageStringTable.getString("FLAboutText");
char const* const _PREHASH_RegionHandshakeReply = gMessageStringTable.getString("RegionHandshakeReply");
char const* const _PREHASH_GroupActiveProposalItemReply = gMessageStringTable.getString("GroupActiveProposalItemReply");
char const* const _PREHASH_MapItemReply = gMessageStringTable.getString("MapItemReply");
char const* const _PREHASH_Seconds = gMessageStringTable.getString("Seconds");
char const* const _PREHASH_UpdateUserInfo = gMessageStringTable.getString("UpdateUserInfo");
char const* const _PREHASH_AggregatePermTexturesOwner = gMessageStringTable.getString("AggregatePermTexturesOwner");
char const* const _PREHASH_Set = gMessageStringTable.getString("Set");
char const* const _PREHASH_NewName = gMessageStringTable.getString("NewName");
char const* const _PREHASH_Key = gMessageStringTable.getString("Key");
char const* const _PREHASH_AgentID = gMessageStringTable.getString("AgentID");
char const* const _PREHASH_EventNotificationRemoveRequest = gMessageStringTable.getString("EventNotificationRemoveRequest");
char const* const _PREHASH_NewFolderID = gMessageStringTable.getString("NewFolderID");
char const* const _PREHASH_Arc = gMessageStringTable.getString("Arc");
char const* const _PREHASH_RegionX = gMessageStringTable.getString("RegionX");
char const* const _PREHASH_RegionY = gMessageStringTable.getString("RegionY");
char const* const _PREHASH_RequestData = gMessageStringTable.getString("RequestData");
char const* const _PREHASH_Msg = gMessageStringTable.getString("Msg");
char const* const _PREHASH_Top = gMessageStringTable.getString("Top");
char const* const _PREHASH_MiscStats = gMessageStringTable.getString("MiscStats");
char const* const _PREHASH_ImageID = gMessageStringTable.getString("ImageID");
char const* const _PREHASH_DataPacket = gMessageStringTable.getString("DataPacket");
char const* const _PREHASH_You = gMessageStringTable.getString("You");
char const* const _PREHASH_ScriptControlChange = gMessageStringTable.getString("ScriptControlChange");
char const* const _PREHASH_LoadURL = gMessageStringTable.getString("LoadURL");
char const* const _PREHASH_SetCPURatio = gMessageStringTable.getString("SetCPURatio");
char const* const _PREHASH_NameValueData = gMessageStringTable.getString("NameValueData");
char const* const _PREHASH_AtomicPassObject = gMessageStringTable.getString("AtomicPassObject");
char const* const _PREHASH_ErrorMessage = gMessageStringTable.getString("ErrorMessage");
char const* const _PREHASH_ViewerFrozenMessage = gMessageStringTable.getString("ViewerFrozenMessage");
char const* const _PREHASH_HealthMessage = gMessageStringTable.getString("HealthMessage");
char const* const _PREHASH_LogTextMessage = gMessageStringTable.getString("LogTextMessage");
char const* const _PREHASH_TimeDilation = gMessageStringTable.getString("TimeDilation");
char const* const _PREHASH_RemoveContribution = gMessageStringTable.getString("RemoveContribution");
char const* const _PREHASH_Contribution = gMessageStringTable.getString("Contribution");
char const* const _PREHASH_SetGroupContribution = gMessageStringTable.getString("SetGroupContribution");
char const* const _PREHASH_Offline = gMessageStringTable.getString("Offline");
char const* const _PREHASH_AgentIsNowWearing = gMessageStringTable.getString("AgentIsNowWearing");
char const* const _PREHASH_Members = gMessageStringTable.getString("Members");
char const* const _PREHASH_FailedResends = gMessageStringTable.getString("FailedResends");
char const* const _PREHASH_SecPerDay = gMessageStringTable.getString("SecPerDay");
char const* const _PREHASH_CameraCenter = gMessageStringTable.getString("CameraCenter");
char const* const _PREHASH_CameraLeftAxis = gMessageStringTable.getString("CameraLeftAxis");
char const* const _PREHASH_ExBlock = gMessageStringTable.getString("ExBlock");
char const* const _PREHASH_Channel = gMessageStringTable.getString("Channel");
char const* const _PREHASH_NetTest = gMessageStringTable.getString("NetTest");
char const* const _PREHASH_DiscardLevel = gMessageStringTable.getString("DiscardLevel");
char const* const _PREHASH_LayerID = gMessageStringTable.getString("LayerID");
char const* const _PREHASH_GrabOffset = gMessageStringTable.getString("GrabOffset");
char const* const _PREHASH_SimPort = gMessageStringTable.getString("SimPort");
char const* const _PREHASH_PricePerMeter = gMessageStringTable.getString("PricePerMeter");
char const* const _PREHASH_RegionFlags = gMessageStringTable.getString("RegionFlags");
char const* const _PREHASH_RegionFlagsExtended = gMessageStringTable.getString("RegionFlagsExtended");
char const* const _PREHASH_RegionProtocols = gMessageStringTable.getString("RegionProtocols");
char const* const _PREHASH_VoteResult = gMessageStringTable.getString("VoteResult");
char const* const _PREHASH_ParcelDirFeeEstimate = gMessageStringTable.getString("ParcelDirFeeEstimate");
char const* const _PREHASH_ModifyBlock = gMessageStringTable.getString("ModifyBlock");
char const* const _PREHASH_InventoryBlock = gMessageStringTable.getString("InventoryBlock");
char const* const _PREHASH_ReplyBlock = gMessageStringTable.getString("ReplyBlock");
char const* const _PREHASH_ValidUntil = gMessageStringTable.getString("ValidUntil");
char const* const _PREHASH_VelocityInterpolateOn = gMessageStringTable.getString("VelocityInterpolateOn");
char const* const _PREHASH_ClassifiedDelete = gMessageStringTable.getString("ClassifiedDelete");
char const* const _PREHASH_RegionDenyAnonymous = gMessageStringTable.getString("RegionDenyAnonymous");
char const* const _PREHASH_FLImageID = gMessageStringTable.getString("FLImageID");
char const* const _PREHASH_AllowPublish = gMessageStringTable.getString("AllowPublish");
char const* const _PREHASH_SitName = gMessageStringTable.getString("SitName");
char const* const _PREHASH_RegionsVisited = gMessageStringTable.getString("RegionsVisited");
char const* const _PREHASH_DirClassifiedReply = gMessageStringTable.getString("DirClassifiedReply");
char const* const _PREHASH_AvatarClassifiedReply = gMessageStringTable.getString("AvatarClassifiedReply");
char const* const _PREHASH_MediaURL = gMessageStringTable.getString("MediaURL");
char const* const _PREHASH_CompleteAgentMovement = gMessageStringTable.getString("CompleteAgentMovement");
char const* const _PREHASH_ClassifiedID = gMessageStringTable.getString("ClassifiedID");
char const* const _PREHASH_LocalID = gMessageStringTable.getString("LocalID");
char const* const _PREHASH_SpaceIP = gMessageStringTable.getString("SpaceIP");
char const* const _PREHASH_RemoveItem = gMessageStringTable.getString("RemoveItem");
char const* const _PREHASH_LogFailedMoneyTransaction = gMessageStringTable.getString("LogFailedMoneyTransaction");
char const* const _PREHASH_ViewerStartAuction = gMessageStringTable.getString("ViewerStartAuction");
char const* const _PREHASH_StartAuction = gMessageStringTable.getString("StartAuction");
char const* const _PREHASH_DuplicateFlags = gMessageStringTable.getString("DuplicateFlags");
char const* const _PREHASH_RegionInfo2 = gMessageStringTable.getString("RegionInfo2");
char const* const _PREHASH_RegionInfo3 = gMessageStringTable.getString("RegionInfo3");
char const* const _PREHASH_RegionInfo4 = gMessageStringTable.getString("RegionInfo4");
char const* const _PREHASH_TextColor = gMessageStringTable.getString("TextColor");
char const* const _PREHASH_SlaveID = gMessageStringTable.getString("SlaveID");
char const* const _PREHASH_Charter = gMessageStringTable.getString("Charter");
char const* const _PREHASH_AlertData = gMessageStringTable.getString("AlertData");
char const* const _PREHASH_AlertInfo = gMessageStringTable.getString("AlertInfo");
char const* const _PREHASH_TargetBlock = gMessageStringTable.getString("TargetBlock");
char const* const _PREHASH_CheckParcelAuctions = gMessageStringTable.getString("CheckParcelAuctions");
char const* const _PREHASH_ParcelAuctions = gMessageStringTable.getString("ParcelAuctions");
char const* const _PREHASH_OwnerIsGroup = gMessageStringTable.getString("OwnerIsGroup");
char const* const _PREHASH_NameValuePair = gMessageStringTable.getString("NameValuePair");
char const* const _PREHASH_RemoveNameValuePair = gMessageStringTable.getString("RemoveNameValuePair");
char const* const _PREHASH_BulkUpdateInventory = gMessageStringTable.getString("BulkUpdateInventory");
char const* const _PREHASH_UpdateTaskInventory = gMessageStringTable.getString("UpdateTaskInventory");
char const* const _PREHASH_RemoveTaskInventory = gMessageStringTable.getString("RemoveTaskInventory");
char const* const _PREHASH_MoveTaskInventory = gMessageStringTable.getString("MoveTaskInventory");
char const* const _PREHASH_RequestTaskInventory = gMessageStringTable.getString("RequestTaskInventory");
char const* const _PREHASH_ReplyTaskInventory = gMessageStringTable.getString("ReplyTaskInventory");
char const* const _PREHASH_AggregatePermInventory = gMessageStringTable.getString("AggregatePermInventory");
char const* const _PREHASH_GroupAccountTransactionsReply = gMessageStringTable.getString("GroupAccountTransactionsReply");
char const* const _PREHASH_SimulatorInfo = gMessageStringTable.getString("SimulatorInfo");
char const* const _PREHASH_WearableData = gMessageStringTable.getString("WearableData");
char const* const _PREHASH_Enabled = gMessageStringTable.getString("Enabled");
char const* const _PREHASH_Savings = gMessageStringTable.getString("Savings");
char const* const _PREHASH_SimulatorLoad = gMessageStringTable.getString("SimulatorLoad");
char const* const _PREHASH_InternalRegionIP = gMessageStringTable.getString("InternalRegionIP");
char const* const _PREHASH_ExternalRegionIP = gMessageStringTable.getString("ExternalRegionIP");
char const* const _PREHASH_TotalPairs = gMessageStringTable.getString("TotalPairs");
char const* const _PREHASH_CreateGroupRequest = gMessageStringTable.getString("CreateGroupRequest");
char const* const _PREHASH_JoinGroupRequest = gMessageStringTable.getString("JoinGroupRequest");
char const* const _PREHASH_LeaveGroupRequest = gMessageStringTable.getString("LeaveGroupRequest");
char const* const _PREHASH_InviteGroupRequest = gMessageStringTable.getString("InviteGroupRequest");
char const* const _PREHASH_LiveHelpGroupRequest = gMessageStringTable.getString("LiveHelpGroupRequest");
char const* const _PREHASH_PriceParcelClaimFactor = gMessageStringTable.getString("PriceParcelClaimFactor");
char const* const _PREHASH_BillableArea = gMessageStringTable.getString("BillableArea");
char const* const _PREHASH_ObjectID = gMessageStringTable.getString("ObjectID");
char const* const _PREHASH_ObjectFlagUpdate = gMessageStringTable.getString("ObjectFlagUpdate");
char const* const _PREHASH_GroupRoleUpdate = gMessageStringTable.getString("GroupRoleUpdate");
char const* const _PREHASH_RequestInventoryAsset = gMessageStringTable.getString("RequestInventoryAsset");
char const* const _PREHASH_ChangedGrid = gMessageStringTable.getString("ChangedGrid");
char const* const _PREHASH_AgentDropGroup = gMessageStringTable.getString("AgentDropGroup");
char const* const _PREHASH_Details = gMessageStringTable.getString("Details");
char const* const _PREHASH_LocationX = gMessageStringTable.getString("LocationX");
char const* const _PREHASH_SaleType = gMessageStringTable.getString("SaleType");
char const* const _PREHASH_LocationY = gMessageStringTable.getString("LocationY");
char const* const _PREHASH_LocationZ = gMessageStringTable.getString("LocationZ");
char const* const _PREHASH_EconomyData = gMessageStringTable.getString("EconomyData");
char const* const _PREHASH_HeadRotation = gMessageStringTable.getString("HeadRotation");
char const* const _PREHASH_DeleteOnCompletion = gMessageStringTable.getString("DeleteOnCompletion");
char const* const _PREHASH_PublicPort = gMessageStringTable.getString("PublicPort");
char const* const _PREHASH_DirClassifiedQuery = gMessageStringTable.getString("DirClassifiedQuery");
char const* const _PREHASH_CallbackID = gMessageStringTable.getString("CallbackID");
char const* const _PREHASH_RequestParcelTransfer = gMessageStringTable.getString("RequestParcelTransfer");
char const* const _PREHASH_RoleCount = gMessageStringTable.getString("RoleCount");
char const* const _PREHASH_RequestID = gMessageStringTable.getString("RequestID");
char const* const _PREHASH_RequestXfer = gMessageStringTable.getString("RequestXfer");
char const* const _PREHASH_ObjectTaxCurrent = gMessageStringTable.getString("ObjectTaxCurrent");
char const* const _PREHASH_LightTaxCurrent = gMessageStringTable.getString("LightTaxCurrent");
char const* const _PREHASH_LandTaxCurrent = gMessageStringTable.getString("LandTaxCurrent");
char const* const _PREHASH_GroupTaxCurrent = gMessageStringTable.getString("GroupTaxCurrent");
char const* const _PREHASH_FetchInventoryDescendents = gMessageStringTable.getString("FetchInventoryDescendents");
char const* const _PREHASH_InventoryDescendents = gMessageStringTable.getString("InventoryDescendents");
char const* const _PREHASH_Descendents = gMessageStringTable.getString("Descendents");
char const* const _PREHASH_PurgeInventoryDescendents = gMessageStringTable.getString("PurgeInventoryDescendents");
char const* const _PREHASH_ShowDir = gMessageStringTable.getString("ShowDir");
char const* const _PREHASH_IsOwner = gMessageStringTable.getString("IsOwner");
char const* const _PREHASH_Timestamp = gMessageStringTable.getString("Timestamp");
char const* const _PREHASH_GlobalPos = gMessageStringTable.getString("GlobalPos");
char const* const _PREHASH_GrabOffsetInitial = gMessageStringTable.getString("GrabOffsetInitial");
char const* const _PREHASH_IsTrial = gMessageStringTable.getString("IsTrial");
char const* const _PREHASH_ObjectDuplicateOnRay = gMessageStringTable.getString("ObjectDuplicateOnRay");
char const* const _PREHASH_GroupMembershipCount = gMessageStringTable.getString("GroupMembershipCount");
char const* const _PREHASH_MethodData = gMessageStringTable.getString("MethodData");
char const* const _PREHASH_ActivateGestures = gMessageStringTable.getString("ActivateGestures");
char const* const _PREHASH_DeactivateGestures = gMessageStringTable.getString("DeactivateGestures");
char const* const _PREHASH_ProposalData = gMessageStringTable.getString("ProposalData");
char const* const _PREHASH_PosGlobal = gMessageStringTable.getString("PosGlobal");
char const* const _PREHASH_SearchID = gMessageStringTable.getString("SearchID");
char const* const _PREHASH_RezMultipleAttachmentsFromInv = gMessageStringTable.getString("RezMultipleAttachmentsFromInv");
char const* const _PREHASH_SearchName = gMessageStringTable.getString("SearchName");
char const* const _PREHASH_VersionString = gMessageStringTable.getString("VersionString");
char const* const _PREHASH_CreateGroupReply = gMessageStringTable.getString("CreateGroupReply");
char const* const _PREHASH_LeaveGroupReply = gMessageStringTable.getString("LeaveGroupReply");
char const* const _PREHASH_ActualArea = gMessageStringTable.getString("ActualArea");
char const* const _PREHASH_Message = gMessageStringTable.getString("Message");
char const* const _PREHASH_ClickAction = gMessageStringTable.getString("ClickAction");
char const* const _PREHASH_AssetUploadComplete = gMessageStringTable.getString("AssetUploadComplete");
char const* const _PREHASH_RequestType = gMessageStringTable.getString("RequestType");
char const* const _PREHASH_UUID = gMessageStringTable.getString("UUID");
char const* const _PREHASH_BaseMask = gMessageStringTable.getString("BaseMask");
char const* const _PREHASH_NetBlock = gMessageStringTable.getString("NetBlock");
char const* const _PREHASH_GlobalX = gMessageStringTable.getString("GlobalX");
char const* const _PREHASH_GlobalY = gMessageStringTable.getString("GlobalY");
char const* const _PREHASH_GlobalZ = gMessageStringTable.getString("GlobalZ");
char const* const _PREHASH_CopyRotates = gMessageStringTable.getString("CopyRotates");
char const* const _PREHASH_KickUserAck = gMessageStringTable.getString("KickUserAck");
char const* const _PREHASH_TopPick = gMessageStringTable.getString("TopPick");
char const* const _PREHASH_SessionID = gMessageStringTable.getString("SessionID");
char const* const _PREHASH_DeclineFriendship = gMessageStringTable.getString("DeclineFriendship");
char const* const _PREHASH_FormFriendship = gMessageStringTable.getString("FormFriendship");
char const* const _PREHASH_TerminateFriendship = gMessageStringTable.getString("TerminateFriendship");
char const* const _PREHASH_TaskData = gMessageStringTable.getString("TaskData");
char const* const _PREHASH_SimWideMaxPrims = gMessageStringTable.getString("SimWideMaxPrims");
char const* const _PREHASH_TotalPrims = gMessageStringTable.getString("TotalPrims");
char const* const _PREHASH_ProfileBegin = gMessageStringTable.getString("ProfileBegin");
char const* const _PREHASH_Request = gMessageStringTable.getString("Request");
char const* const _PREHASH_GroupAccountDetailsRequest = gMessageStringTable.getString("GroupAccountDetailsRequest");
char const* const _PREHASH_GroupActiveProposalsRequest = gMessageStringTable.getString("GroupActiveProposalsRequest");
char const* const _PREHASH_StringValue = gMessageStringTable.getString("StringValue");
char const* const _PREHASH_Version = gMessageStringTable.getString("Version");
char const* const _PREHASH_OtherCount = gMessageStringTable.getString("OtherCount");
char const* const _PREHASH_MemberCount = gMessageStringTable.getString("MemberCount");
char const* const _PREHASH_ChatData = gMessageStringTable.getString("ChatData");
char const* const _PREHASH_IsGroupOwned = gMessageStringTable.getString("IsGroupOwned");
char const* const _PREHASH_EnergyEfficiency = gMessageStringTable.getString("EnergyEfficiency");
char const* const _PREHASH_PickInfoUpdate = gMessageStringTable.getString("PickInfoUpdate");
char const* const _PREHASH_PickDelete = gMessageStringTable.getString("PickDelete");
char const* const _PREHASH_ScriptReset = gMessageStringTable.getString("ScriptReset");
char const* const _PREHASH_Requester = gMessageStringTable.getString("Requester");
char const* const _PREHASH_ForSale = gMessageStringTable.getString("ForSale");
char const* const _PREHASH_NearestLandingRegionReply = gMessageStringTable.getString("NearestLandingRegionReply");
char const* const _PREHASH_ParcelID = gMessageStringTable.getString("ParcelID");
char const* const _PREHASH_Godlike = gMessageStringTable.getString("Godlike");
char const* const _PREHASH_TotalDebits = gMessageStringTable.getString("TotalDebits");
char const* const _PREHASH_Direction = gMessageStringTable.getString("Direction");
char const* const _PREHASH_HealthData = gMessageStringTable.getString("HealthData");
char const* const _PREHASH_LeftAxis = gMessageStringTable.getString("LeftAxis");
char const* const _PREHASH_LocationBlock = gMessageStringTable.getString("LocationBlock");
char const* const _PREHASH_ObjectImage = gMessageStringTable.getString("ObjectImage");
char const* const _PREHASH_TerrainStartHeight00 = gMessageStringTable.getString("TerrainStartHeight00");
char const* const _PREHASH_TerrainStartHeight01 = gMessageStringTable.getString("TerrainStartHeight01");
char const* const _PREHASH_TerrainStartHeight10 = gMessageStringTable.getString("TerrainStartHeight10");
char const* const _PREHASH_TerrainStartHeight11 = gMessageStringTable.getString("TerrainStartHeight11");
char const* const _PREHASH_WaterHeight = gMessageStringTable.getString("WaterHeight");
char const* const _PREHASH_FetchInventoryReply = gMessageStringTable.getString("FetchInventoryReply");
char const* const _PREHASH_GroupAccountSummaryReply = gMessageStringTable.getString("GroupAccountSummaryReply");
char const* const _PREHASH_AttachedSound = gMessageStringTable.getString("AttachedSound");
char const* const _PREHASH_ParamInUse = gMessageStringTable.getString("ParamInUse");
char const* const _PREHASH_GodKickUser = gMessageStringTable.getString("GodKickUser");
char const* const _PREHASH_PickName = gMessageStringTable.getString("PickName");
char const* const _PREHASH_TaskName = gMessageStringTable.getString("TaskName");
char const* const _PREHASH_RegionPresenceRequestByHandle = gMessageStringTable.getString("RegionPresenceRequestByHandle");
char const* const _PREHASH_RezSingleAttachmentFromInv = gMessageStringTable.getString("RezSingleAttachmentFromInv");
char const* const _PREHASH_ChildAgentUpdate = gMessageStringTable.getString("ChildAgentUpdate");
char const* const _PREHASH_IsOwnerGroup = gMessageStringTable.getString("IsOwnerGroup");
char const* const _PREHASH_AgentHeightWidth = gMessageStringTable.getString("AgentHeightWidth");
char const* const _PREHASH_VerticalAngle = gMessageStringTable.getString("VerticalAngle");
char const* const _PREHASH_WearableType = gMessageStringTable.getString("WearableType");
char const* const _PREHASH_AggregatePermNextOwner = gMessageStringTable.getString("AggregatePermNextOwner");
char const* const _PREHASH_ShowInList = gMessageStringTable.getString("ShowInList");
char const* const _PREHASH_UpdateParcel = gMessageStringTable.getString("UpdateParcel");
char const* const _PREHASH_SetAlwaysRun = gMessageStringTable.getString("SetAlwaysRun");
char const* const _PREHASH_NVPair = gMessageStringTable.getString("NVPair");
char const* const _PREHASH_SearchType = gMessageStringTable.getString("SearchType");
char const* const _PREHASH_ObjectSpinStart = gMessageStringTable.getString("ObjectSpinStart");
char const* const _PREHASH_UseEstateSun = gMessageStringTable.getString("UseEstateSun");
char const* const _PREHASH_RegionID = gMessageStringTable.getString("RegionID");
char const* const _PREHASH_AbuseRegionID = gMessageStringTable.getString("AbuseRegionID");
char const* const _PREHASH_Creator = gMessageStringTable.getString("Creator");
char const* const _PREHASH_ProposalText = gMessageStringTable.getString("ProposalText");
char const* const _PREHASH_DirEventsReply = gMessageStringTable.getString("DirEventsReply");
char const* const _PREHASH_EventInfoReply = gMessageStringTable.getString("EventInfoReply");
char const* const _PREHASH_UserInfoReply = gMessageStringTable.getString("UserInfoReply");
char const* const _PREHASH_PathRadiusOffset = gMessageStringTable.getString("PathRadiusOffset");
char const* const _PREHASH_TextureData = gMessageStringTable.getString("TextureData");
char const* const _PREHASH_ChatPass = gMessageStringTable.getString("ChatPass");
char const* const _PREHASH_TargetID = gMessageStringTable.getString("TargetID");
char const* const _PREHASH_DefaultPayPrice = gMessageStringTable.getString("DefaultPayPrice");
char const* const _PREHASH_UserLocation = gMessageStringTable.getString("UserLocation");
char const* const _PREHASH_MaxPrims = gMessageStringTable.getString("MaxPrims");
char const* const _PREHASH_LandmarkID = gMessageStringTable.getString("LandmarkID");
char const* const _PREHASH_InitiateDownload = gMessageStringTable.getString("InitiateDownload");
char const* const _PREHASH_Name = gMessageStringTable.getString("Name");
char const* const _PREHASH_OtherCleanTime = gMessageStringTable.getString("OtherCleanTime");
char const* const _PREHASH_ParcelSetOtherCleanTime = gMessageStringTable.getString("ParcelSetOtherCleanTime");
char const* const _PREHASH_TeleportPriceExponent = gMessageStringTable.getString("TeleportPriceExponent");
char const* const _PREHASH_Gain = gMessageStringTable.getString("Gain");
char const* const _PREHASH_PacketAck = gMessageStringTable.getString("PacketAck");
char const* const _PREHASH_PathSkew = gMessageStringTable.getString("PathSkew");
char const* const _PREHASH_SimulatorShutdownRequest = gMessageStringTable.getString("SimulatorShutdownRequest");
char const* const _PREHASH_NearestLandingRegionRequest = gMessageStringTable.getString("NearestLandingRegionRequest");
char const* const _PREHASH_OtherID = gMessageStringTable.getString("OtherID");
char const* const _PREHASH_MemberID = gMessageStringTable.getString("MemberID");
char const* const _PREHASH_MapLayerRequest = gMessageStringTable.getString("MapLayerRequest");
char const* const _PREHASH_ObjectScale = gMessageStringTable.getString("ObjectScale");
char const* const _PREHASH_TargetIP = gMessageStringTable.getString("TargetIP");
char const* const _PREHASH_Redo = gMessageStringTable.getString("Redo");
char const* const _PREHASH_MoneyBalance = gMessageStringTable.getString("MoneyBalance");
char const* const _PREHASH_TrackAgent = gMessageStringTable.getString("TrackAgent");
char const* const _PREHASH_MaxX = gMessageStringTable.getString("MaxX");
char const* const _PREHASH_MaxY = gMessageStringTable.getString("MaxY");
char const* const _PREHASH_Data = gMessageStringTable.getString("Data");
char const* const _PREHASH_NewData = gMessageStringTable.getString("NewData");
char const* const _PREHASH_ListInProfile = gMessageStringTable.getString("ListInProfile");
char const* const _PREHASH_TextureAnim = gMessageStringTable.getString("TextureAnim");
char const* const _PREHASH_ReturnIDs = gMessageStringTable.getString("ReturnIDs");
char const* const _PREHASH_Date = gMessageStringTable.getString("Date");
char const* const _PREHASH_AgentWearablesUpdate = gMessageStringTable.getString("AgentWearablesUpdate");
char const* const _PREHASH_AgentDataUpdate = gMessageStringTable.getString("AgentDataUpdate");
char const* const _PREHASH_GroupDataUpdate = gMessageStringTable.getString("GroupDataUpdate");
char const* const _PREHASH_Hash = gMessageStringTable.getString("Hash");
char const* const _PREHASH_AgentGroupDataUpdate = gMessageStringTable.getString("AgentGroupDataUpdate");
char const* const _PREHASH_Left = gMessageStringTable.getString("Left");
char const* const _PREHASH_Mask = gMessageStringTable.getString("Mask");
char const* const _PREHASH_ForceMouselook = gMessageStringTable.getString("ForceMouselook");
char const* const _PREHASH_Success = gMessageStringTable.getString("Success");
char const* const _PREHASH_ObjectGroup = gMessageStringTable.getString("ObjectGroup");
char const* const _PREHASH_SunHour = gMessageStringTable.getString("SunHour");
char const* const _PREHASH_MinX = gMessageStringTable.getString("MinX");
char const* const _PREHASH_ScriptSensorReply = gMessageStringTable.getString("ScriptSensorReply");
char const* const _PREHASH_MinY = gMessageStringTable.getString("MinY");
char const* const _PREHASH_Command = gMessageStringTable.getString("Command");
char const* const _PREHASH_Desc = gMessageStringTable.getString("Desc");
char const* const _PREHASH_AttachmentNeedsSave = gMessageStringTable.getString("AttachmentNeedsSave");
char const* const _PREHASH_HistoryItemData = gMessageStringTable.getString("HistoryItemData");
char const* const _PREHASH_AgentCachedTexture = gMessageStringTable.getString("AgentCachedTexture");
char const* const _PREHASH_Subject = gMessageStringTable.getString("Subject");
char const* const _PREHASH_QueryReplies = gMessageStringTable.getString("QueryReplies");
char const* const _PREHASH_ObjectCategory = gMessageStringTable.getString("ObjectCategory");
char const* const _PREHASH_Time = gMessageStringTable.getString("Time");
char const* const _PREHASH_CreateLandmarkForEvent = gMessageStringTable.getString("CreateLandmarkForEvent");
char const* const _PREHASH_ParentID = gMessageStringTable.getString("ParentID");
char const* const _PREHASH_Ping = gMessageStringTable.getString("Ping");
char const* const _PREHASH_Perp = gMessageStringTable.getString("Perp");
char const* const _PREHASH_Code = gMessageStringTable.getString("Code");
char const* const _PREHASH_InvType = gMessageStringTable.getString("InvType");
char const* const _PREHASH_AgentFOV = gMessageStringTable.getString("AgentFOV");
char const* const _PREHASH_Audible = gMessageStringTable.getString("Audible");
char const* const _PREHASH_AuctionData = gMessageStringTable.getString("AuctionData");
char const* const _PREHASH_IDBlock = gMessageStringTable.getString("IDBlock");
char const* const _PREHASH_North = gMessageStringTable.getString("North");
char const* const _PREHASH_South = gMessageStringTable.getString("South");
char const* const _PREHASH_West = gMessageStringTable.getString("West");
char const* const _PREHASH_East = gMessageStringTable.getString("East");
char const* const _PREHASH_Undo = gMessageStringTable.getString("Undo");
char const* const _PREHASH_TotalNumItems = gMessageStringTable.getString("TotalNumItems");
char const* const _PREHASH_Info = gMessageStringTable.getString("Info");
char const* const _PREHASH_Area = gMessageStringTable.getString("Area");
char const* const _PREHASH_SimCrashed = gMessageStringTable.getString("SimCrashed");
char const* const _PREHASH_Text = gMessageStringTable.getString("Text");
char const* const _PREHASH_PriceGroupCreate = gMessageStringTable.getString("PriceGroupCreate");
char const* const _PREHASH_ObjectShape = gMessageStringTable.getString("ObjectShape");
char const* const _PREHASH_GroupRoleDataReply = gMessageStringTable.getString("GroupRoleDataReply");
char const* const _PREHASH_MuteCRC = gMessageStringTable.getString("MuteCRC");
char const* const _PREHASH_Size = gMessageStringTable.getString("Size");
char const* const _PREHASH_FromAddress = gMessageStringTable.getString("FromAddress");
char const* const _PREHASH_Body = gMessageStringTable.getString("Body");
char const* const _PREHASH_FileData = gMessageStringTable.getString("FileData");
char const* const _PREHASH_List = gMessageStringTable.getString("List");
char const* const _PREHASH_KickUser = gMessageStringTable.getString("KickUser");
char const* const _PREHASH_OtherPrims = gMessageStringTable.getString("OtherPrims");
char const* const _PREHASH_RunTime = gMessageStringTable.getString("RunTime");
char const* const _PREHASH_GrantUserRights = gMessageStringTable.getString("GrantUserRights");
char const* const _PREHASH_RpcScriptRequestInboundForward = gMessageStringTable.getString("RpcScriptRequestInboundForward");
char const* const _PREHASH_More = gMessageStringTable.getString("More");
char const* const _PREHASH_Majority = gMessageStringTable.getString("Majority");
char const* const _PREHASH_MetersTraveled = gMessageStringTable.getString("MetersTraveled");
char const* const _PREHASH_Stat = gMessageStringTable.getString("Stat");
char const* const _PREHASH_SoundID = gMessageStringTable.getString("SoundID");
char const* const _PREHASH_Item = gMessageStringTable.getString("Item");
char const* const _PREHASH_User = gMessageStringTable.getString("User");
char const* const _PREHASH_Prey = gMessageStringTable.getString("Prey");
char const* const _PREHASH_RayStart = gMessageStringTable.getString("RayStart");
char const* const _PREHASH_UsecSinceStart = gMessageStringTable.getString("UsecSinceStart");
char const* const _PREHASH_ParcelData = gMessageStringTable.getString("ParcelData");
char const* const _PREHASH_CameraUpAxis = gMessageStringTable.getString("CameraUpAxis");
char const* const _PREHASH_ScriptDialog = gMessageStringTable.getString("ScriptDialog");
char const* const _PREHASH_MasterParcelData = gMessageStringTable.getString("MasterParcelData");
char const* const _PREHASH_Invalid = gMessageStringTable.getString("Invalid");
char const* const _PREHASH_ProfileCurve = gMessageStringTable.getString("ProfileCurve");
char const* const _PREHASH_ParcelAccessListUpdate = gMessageStringTable.getString("ParcelAccessListUpdate");
char const* const _PREHASH_MuteListUpdate = gMessageStringTable.getString("MuteListUpdate");
char const* const _PREHASH_SendPacket = gMessageStringTable.getString("SendPacket");
char const* const _PREHASH_SendXferPacket = gMessageStringTable.getString("SendXferPacket");
char const* const _PREHASH_RegionDenyIdentified = gMessageStringTable.getString("RegionDenyIdentified");
char const* const _PREHASH_NotecardItemID = gMessageStringTable.getString("NotecardItemID");
char const* const _PREHASH_LastName = gMessageStringTable.getString("LastName");
char const* const _PREHASH_From = gMessageStringTable.getString("From");
char const* const _PREHASH_RoleChange = gMessageStringTable.getString("RoleChange");
char const* const _PREHASH_Port = gMessageStringTable.getString("Port");
char const* const _PREHASH_MemberTitle = gMessageStringTable.getString("MemberTitle");
char const* const _PREHASH_LogParcelChanges = gMessageStringTable.getString("LogParcelChanges");
char const* const _PREHASH_AgentCachedTextureResponse = gMessageStringTable.getString("AgentCachedTextureResponse");
char const* const _PREHASH_DeRezObject = gMessageStringTable.getString("DeRezObject");
char const* const _PREHASH_IsTemporary = gMessageStringTable.getString("IsTemporary");
char const* const _PREHASH_IsPhantom = gMessageStringTable.getString("IsPhantom");
char const* const _PREHASH_CastsShadows = gMessageStringTable.getString("CastsShadows");
char const* const _PREHASH_ExtraPhysics = gMessageStringTable.getString("ExtraPhysics");
char const* const _PREHASH_PhysicsShapeType = gMessageStringTable.getString("PhysicsShapeType");
char const* const _PREHASH_Density = gMessageStringTable.getString("Density");
char const* const _PREHASH_Friction = gMessageStringTable.getString("Friction");
char const* const _PREHASH_Restitution = gMessageStringTable.getString("Restitution");
char const* const _PREHASH_GravityMultiplier = gMessageStringTable.getString("GravityMultiplier");
char const* const _PREHASH_InsigniaID = gMessageStringTable.getString("InsigniaID");
char const* const _PREHASH_CheckFlags = gMessageStringTable.getString("CheckFlags");
char const* const _PREHASH_EventID = gMessageStringTable.getString("EventID");
char const* const _PREHASH_Selected = gMessageStringTable.getString("Selected");
char const* const _PREHASH_FromAgentId = gMessageStringTable.getString("FromAgentId");
char const* const _PREHASH_Type = gMessageStringTable.getString("Type");
char const* const _PREHASH_ChatType = gMessageStringTable.getString("ChatType");
char const* const _PREHASH_ReportData = gMessageStringTable.getString("ReportData");
char const* const _PREHASH_RequestBlock = gMessageStringTable.getString("RequestBlock");
char const* const _PREHASH_GrantData = gMessageStringTable.getString("GrantData");
char const* const _PREHASH_DetachAttachmentIntoInv = gMessageStringTable.getString("DetachAttachmentIntoInv");
char const* const _PREHASH_ParcelDisableObjects = gMessageStringTable.getString("ParcelDisableObjects");
char const* const _PREHASH_Sections = gMessageStringTable.getString("Sections");
char const* const _PREHASH_GodLevel = gMessageStringTable.getString("GodLevel");
char const* const _PREHASH_PayPriceReply = gMessageStringTable.getString("PayPriceReply");
char const* const _PREHASH_QueryID = gMessageStringTable.getString("QueryID");
char const* const _PREHASH_CameraEyeOffset = gMessageStringTable.getString("CameraEyeOffset");
char const* const _PREHASH_AgentPosition = gMessageStringTable.getString("AgentPosition");
char const* const _PREHASH_GrabPosition = gMessageStringTable.getString("GrabPosition");
char const* const _PREHASH_OnlineNotification = gMessageStringTable.getString("OnlineNotification");
char const* const _PREHASH_OfflineNotification = gMessageStringTable.getString("OfflineNotification");
char const* const _PREHASH_SendPostcard = gMessageStringTable.getString("SendPostcard");
char const* const _PREHASH_RequestFlags = gMessageStringTable.getString("RequestFlags");
char const* const _PREHASH_GroupAccountSummaryRequest = gMessageStringTable.getString("GroupAccountSummaryRequest");
char const* const _PREHASH_GroupVoteHistoryRequest = gMessageStringTable.getString("GroupVoteHistoryRequest");
char const* const _PREHASH_ParamValue = gMessageStringTable.getString("ParamValue");
char const* const _PREHASH_MaxAgents = gMessageStringTable.getString("MaxAgents");
char const* const _PREHASH_HardMaxAgents = gMessageStringTable.getString("HardMaxAgents");
char const* const _PREHASH_CreateNewOutfitAttachments = gMessageStringTable.getString("CreateNewOutfitAttachments");
char const* const _PREHASH_RegionHandle = gMessageStringTable.getString("RegionHandle");
char const* const _PREHASH_ChannelVersion = gMessageStringTable.getString("ChannelVersion");
char const* const _PREHASH_TeleportProgress = gMessageStringTable.getString("TeleportProgress");
char const* const _PREHASH_AgentQuitCopy = gMessageStringTable.getString("AgentQuitCopy");
char const* const _PREHASH_AvatarInterestsUpdate = gMessageStringTable.getString("AvatarInterestsUpdate");
char const* const _PREHASH_GroupNoticeID = gMessageStringTable.getString("GroupNoticeID");
char const* const _PREHASH_ParcelName = gMessageStringTable.getString("ParcelName");
char const* const _PREHASH_PriceObjectRent = gMessageStringTable.getString("PriceObjectRent");
char const* const _PREHASH_OfferCallingCard = gMessageStringTable.getString("OfferCallingCard");
char const* const _PREHASH_AcceptCallingCard = gMessageStringTable.getString("AcceptCallingCard");
char const* const _PREHASH_DeclineCallingCard = gMessageStringTable.getString("DeclineCallingCard");
char const* const _PREHASH_AgentAccess = gMessageStringTable.getString("AgentAccess");
char const* const _PREHASH_AgentLegacyAccess = gMessageStringTable.getString("AgentLegacyAccess");
char const* const _PREHASH_AgentMaxAccess = gMessageStringTable.getString("AgentMaxAccess");
char const* const _PREHASH_DataHomeLocationReply = gMessageStringTable.getString("DataHomeLocationReply");
char const* const _PREHASH_EventLocationReply = gMessageStringTable.getString("EventLocationReply");
char const* const _PREHASH_TerseDateID = gMessageStringTable.getString("TerseDateID");
char const* const _PREHASH_ObjectOwner = gMessageStringTable.getString("ObjectOwner");
char const* const _PREHASH_AssetID = gMessageStringTable.getString("AssetID");
char const* const _PREHASH_AlertMessage = gMessageStringTable.getString("AlertMessage");
char const* const _PREHASH_AgentAlertMessage = gMessageStringTable.getString("AgentAlertMessage");
char const* const _PREHASH_EstateOwnerMessage = gMessageStringTable.getString("EstateOwnerMessage");
char const* const _PREHASH_ParcelMediaCommandMessage = gMessageStringTable.getString("ParcelMediaCommandMessage");
char const* const _PREHASH_Auction = gMessageStringTable.getString("Auction");
char const* const _PREHASH_Category = gMessageStringTable.getString("Category");
char const* const _PREHASH_FilePath = gMessageStringTable.getString("FilePath");
char const* const _PREHASH_ItemFlags = gMessageStringTable.getString("ItemFlags");
char const* const _PREHASH_Invoice = gMessageStringTable.getString("Invoice");
char const* const _PREHASH_IntervalDays = gMessageStringTable.getString("IntervalDays");
char const* const _PREHASH_PathScaleX = gMessageStringTable.getString("PathScaleX");
char const* const _PREHASH_FromTaskID = gMessageStringTable.getString("FromTaskID");
char const* const _PREHASH_PathScaleY = gMessageStringTable.getString("PathScaleY");
char const* const _PREHASH_TimeInfo = gMessageStringTable.getString("TimeInfo");
char const* const _PREHASH_PublicCount = gMessageStringTable.getString("PublicCount");
char const* const _PREHASH_ParcelJoin = gMessageStringTable.getString("ParcelJoin");
char const* const _PREHASH_GroupRolesCount = gMessageStringTable.getString("GroupRolesCount");
char const* const _PREHASH_SimulatorBlock = gMessageStringTable.getString("SimulatorBlock");
char const* const _PREHASH_GroupID = gMessageStringTable.getString("GroupID");
char const* const _PREHASH_AgentVel = gMessageStringTable.getString("AgentVel");
char const* const _PREHASH_RequestImage = gMessageStringTable.getString("RequestImage");
char const* const _PREHASH_NetStats = gMessageStringTable.getString("NetStats");
char const* const _PREHASH_AgentPos = gMessageStringTable.getString("AgentPos");
char const* const _PREHASH_AgentSit = gMessageStringTable.getString("AgentSit");
char const* const _PREHASH_Material = gMessageStringTable.getString("Material");
char const* const _PREHASH_ObjectDeGrab = gMessageStringTable.getString("ObjectDeGrab");
char const* const _PREHASH_VelocityInterpolateOff = gMessageStringTable.getString("VelocityInterpolateOff");
char const* const _PREHASH_AuthorizedBuyerID = gMessageStringTable.getString("AuthorizedBuyerID");
char const* const _PREHASH_AvatarPropertiesReply = gMessageStringTable.getString("AvatarPropertiesReply");
char const* const _PREHASH_GroupProfileReply = gMessageStringTable.getString("GroupProfileReply");
char const* const _PREHASH_SimOwner = gMessageStringTable.getString("SimOwner");
char const* const _PREHASH_SalePrice = gMessageStringTable.getString("SalePrice");
char const* const _PREHASH_Animation = gMessageStringTable.getString("Animation");
char const* const _PREHASH_OwnerID = gMessageStringTable.getString("OwnerID");
char const* const _PREHASH_NearestLandingRegionUpdated = gMessageStringTable.getString("NearestLandingRegionUpdated");
char const* const _PREHASH_PassToAgent = gMessageStringTable.getString("PassToAgent");
char const* const _PREHASH_PreyAgent = gMessageStringTable.getString("PreyAgent");
char const* const _PREHASH_SimStats = gMessageStringTable.getString("SimStats");
char const* const _PREHASH_LogoutReply = gMessageStringTable.getString("LogoutReply");
char const* const _PREHASH_FeatureDisabled = gMessageStringTable.getString("FeatureDisabled");
char const* const _PREHASH_PhysicalAvatarEventList = gMessageStringTable.getString("PhysicalAvatarEventList");
char const* const _PREHASH_ObjectLocalID = gMessageStringTable.getString("ObjectLocalID");
char const* const _PREHASH_Dropped = gMessageStringTable.getString("Dropped");
char const* const _PREHASH_WebProfilesDisabled = gMessageStringTable.getString("WebProfilesDisabled");
char const* const _PREHASH_Destination = gMessageStringTable.getString("Destination");
char const* const _PREHASH_MasterID = gMessageStringTable.getString("MasterID");
char const* const _PREHASH_TransferData = gMessageStringTable.getString("TransferData");
char const* const _PREHASH_WantToMask = gMessageStringTable.getString("WantToMask");
char const* const _PREHASH_ParcelSelectObjects = gMessageStringTable.getString("ParcelSelectObjects");
char const* const _PREHASH_ExtraParams = gMessageStringTable.getString("ExtraParams");
char const* const _PREHASH_CreatorID = gMessageStringTable.getString("CreatorID");
char const* const _PREHASH_Summary = gMessageStringTable.getString("Summary");
char const* const _PREHASH_BuyObjectInventory = gMessageStringTable.getString("BuyObjectInventory");
char const* const _PREHASH_FetchInventory = gMessageStringTable.getString("FetchInventory");
char const* const _PREHASH_InventoryID = gMessageStringTable.getString("InventoryID");
char const* const _PREHASH_PacketNumber = gMessageStringTable.getString("PacketNumber");
char const* const _PREHASH_SetFollowCamProperties = gMessageStringTable.getString("SetFollowCamProperties");
char const* const _PREHASH_ClearFollowCamProperties = gMessageStringTable.getString("ClearFollowCamProperties");
char const* const _PREHASH_SequenceID = gMessageStringTable.getString("SequenceID");
char const* const _PREHASH_DataServerLogout = gMessageStringTable.getString("DataServerLogout");
char const* const _PREHASH_NameValue = gMessageStringTable.getString("NameValue");
char const* const _PREHASH_PathShearX = gMessageStringTable.getString("PathShearX");
char const* const _PREHASH_PathShearY = gMessageStringTable.getString("PathShearY");
char const* const _PREHASH_Velocity = gMessageStringTable.getString("Velocity");
char const* const _PREHASH_SecPerYear = gMessageStringTable.getString("SecPerYear");
char const* const _PREHASH_FirstName = gMessageStringTable.getString("FirstName");
char const* const _PREHASH_AttachedSoundGainChange = gMessageStringTable.getString("AttachedSoundGainChange");
char const* const _PREHASH_LocationID = gMessageStringTable.getString("LocationID");
char const* const _PREHASH_Running = gMessageStringTable.getString("Running");
char const* const _PREHASH_Mono = gMessageStringTable.getString("Mono");
char const* const _PREHASH_Luau = gMessageStringTable.getString("Luau");
char const* const _PREHASH_LuauLanguage = gMessageStringTable.getString("LuauLanguage");
char const* const _PREHASH_AgentThrottle = gMessageStringTable.getString("AgentThrottle");
char const* const _PREHASH_NeighborList = gMessageStringTable.getString("NeighborList");
char const* const _PREHASH_PathTaperX = gMessageStringTable.getString("PathTaperX");
char const* const _PREHASH_PathTaperY = gMessageStringTable.getString("PathTaperY");
char const* const _PREHASH_AgentRelated = gMessageStringTable.getString("AgentRelated");
char const* const _PREHASH_GranterBlock = gMessageStringTable.getString("GranterBlock");
char const* const _PREHASH_UseCachedMuteList = gMessageStringTable.getString("UseCachedMuteList");
char const* const _PREHASH_FailStats = gMessageStringTable.getString("FailStats");
char const* const _PREHASH_Tempfile = gMessageStringTable.getString("Tempfile");
char const* const _PREHASH_BuyerID = gMessageStringTable.getString("BuyerID");
char const* const _PREHASH_DirPeopleReply = gMessageStringTable.getString("DirPeopleReply");
char const* const _PREHASH_TransferInfo = gMessageStringTable.getString("TransferInfo");
char const* const _PREHASH_AvatarPickerRequestBackend = gMessageStringTable.getString("AvatarPickerRequestBackend");
char const* const _PREHASH_AvatarPropertiesRequestBackend = gMessageStringTable.getString("AvatarPropertiesRequestBackend");
char const* const _PREHASH_UpdateData = gMessageStringTable.getString("UpdateData");
char const* const _PREHASH_SimFPS = gMessageStringTable.getString("SimFPS");
char const* const _PREHASH_ReporterID = gMessageStringTable.getString("ReporterID");
char const* const _PREHASH_ButtonLabel = gMessageStringTable.getString("ButtonLabel");
char const* const _PREHASH_GranterID = gMessageStringTable.getString("GranterID");
char const* const _PREHASH_WantToText = gMessageStringTable.getString("WantToText");
char const* const _PREHASH_ReportType = gMessageStringTable.getString("ReportType");
char const* const _PREHASH_SimulatorReady = gMessageStringTable.getString("SimulatorReady");
char const* const _PREHASH_DataBlock = gMessageStringTable.getString("DataBlock");
char const* const _PREHASH_AnimationSourceList = gMessageStringTable.getString("AnimationSourceList");
char const* const _PREHASH_SubscribeLoad = gMessageStringTable.getString("SubscribeLoad");
char const* const _PREHASH_UnsubscribeLoad = gMessageStringTable.getString("UnsubscribeLoad");
char const* const _PREHASH_Packet = gMessageStringTable.getString("Packet");
char const* const _PREHASH_UndoLand = gMessageStringTable.getString("UndoLand");
char const* const _PREHASH_SimAccess = gMessageStringTable.getString("SimAccess");
char const* const _PREHASH_AbuserID = gMessageStringTable.getString("AbuserID");
char const* const _PREHASH_MembershipFee = gMessageStringTable.getString("MembershipFee");
char const* const _PREHASH_InviteGroupResponse = gMessageStringTable.getString("InviteGroupResponse");
char const* const _PREHASH_CreateInventoryFolder = gMessageStringTable.getString("CreateInventoryFolder");
char const* const _PREHASH_UpdateInventoryFolder = gMessageStringTable.getString("UpdateInventoryFolder");
char const* const _PREHASH_MoveInventoryFolder = gMessageStringTable.getString("MoveInventoryFolder");
char const* const _PREHASH_RemoveInventoryFolder = gMessageStringTable.getString("RemoveInventoryFolder");
char const* const _PREHASH_MoneyData = gMessageStringTable.getString("MoneyData");
char const* const _PREHASH_ObjectDeselect = gMessageStringTable.getString("ObjectDeselect");
char const* const _PREHASH_NewAssetID = gMessageStringTable.getString("NewAssetID");
char const* const _PREHASH_ObjectAdd = gMessageStringTable.getString("ObjectAdd");
char const* const _PREHASH_RayEndIsIntersection = gMessageStringTable.getString("RayEndIsIntersection");
char const* const _PREHASH_CompleteAuction = gMessageStringTable.getString("CompleteAuction");
char const* const _PREHASH_CircuitCode = gMessageStringTable.getString("CircuitCode");
char const* const _PREHASH_AgentMovementComplete = gMessageStringTable.getString("AgentMovementComplete");
char const* const _PREHASH_ViewerIP = gMessageStringTable.getString("ViewerIP");
char const* const _PREHASH_Header = gMessageStringTable.getString("Header");
char const* const _PREHASH_GestureFlags = gMessageStringTable.getString("GestureFlags");
char const* const _PREHASH_XferID = gMessageStringTable.getString("XferID");
char const* const _PREHASH_StatValue = gMessageStringTable.getString("StatValue");
char const* const _PREHASH_TaskID = gMessageStringTable.getString("TaskID");
char const* const _PREHASH_PickID = gMessageStringTable.getString("PickID");
char const* const _PREHASH_RayEnd = gMessageStringTable.getString("RayEnd");
char const* const _PREHASH_Throttles = gMessageStringTable.getString("Throttles");
char const* const _PREHASH_RebakeAvatarTextures = gMessageStringTable.getString("RebakeAvatarTextures");
char const* const _PREHASH_UpAxis = gMessageStringTable.getString("UpAxis");
char const* const _PREHASH_AgentTextures = gMessageStringTable.getString("AgentTextures");
char const* const _PREHASH_NotecardData = gMessageStringTable.getString("NotecardData");
char const* const _PREHASH_Radius = gMessageStringTable.getString("Radius");
char const* const _PREHASH_OffCircuit = gMessageStringTable.getString("OffCircuit");
char const* const _PREHASH_Access = gMessageStringTable.getString("Access");
char const* const _PREHASH_TitleRoleID = gMessageStringTable.getString("TitleRoleID");
char const* const _PREHASH_SquareMetersCredit = gMessageStringTable.getString("SquareMetersCredit");
char const* const _PREHASH_Filename = gMessageStringTable.getString("Filename");
char const* const _PREHASH_ClassifiedInfoRequest = gMessageStringTable.getString("ClassifiedInfoRequest");
char const* const _PREHASH_ParcelInfoRequest = gMessageStringTable.getString("ParcelInfoRequest");
char const* const _PREHASH_ParcelObjectOwnersRequest = gMessageStringTable.getString("ParcelObjectOwnersRequest");
char const* const _PREHASH_TeleportLandmarkRequest = gMessageStringTable.getString("TeleportLandmarkRequest");
char const* const _PREHASH_EventInfoRequest = gMessageStringTable.getString("EventInfoRequest");
char const* const _PREHASH_MoneyBalanceRequest = gMessageStringTable.getString("MoneyBalanceRequest");
char const* const _PREHASH_GroupMembersRequest = gMessageStringTable.getString("GroupMembersRequest");
char const* const _PREHASH_GroupRoleMembersRequest = gMessageStringTable.getString("GroupRoleMembersRequest");
char const* const _PREHASH_ChatFromSimulator = gMessageStringTable.getString("ChatFromSimulator");
char const* const _PREHASH_OldFolderID = gMessageStringTable.getString("OldFolderID");
char const* const _PREHASH_UserInfoRequest = gMessageStringTable.getString("UserInfoRequest");
char const* const _PREHASH_TextureID = gMessageStringTable.getString("TextureID");
char const* const _PREHASH_ProfileURL = gMessageStringTable.getString("ProfileURL");
char const* const _PREHASH_Handle = gMessageStringTable.getString("Handle");
char const* const _PREHASH_ButtonIndex = gMessageStringTable.getString("ButtonIndex");
char const* const _PREHASH_GetScriptRunning = gMessageStringTable.getString("GetScriptRunning");
char const* const _PREHASH_SetScriptRunning = gMessageStringTable.getString("SetScriptRunning");
char const* const _PREHASH_Health = gMessageStringTable.getString("Health");
char const* const _PREHASH_CircuitInfo = gMessageStringTable.getString("CircuitInfo");
char const* const _PREHASH_ObjectBuy = gMessageStringTable.getString("ObjectBuy");
char const* const _PREHASH_ProfileEnd = gMessageStringTable.getString("ProfileEnd");
char const* const _PREHASH_Effect = gMessageStringTable.getString("Effect");
char const* const _PREHASH_TestMessage = gMessageStringTable.getString("TestMessage");
char const* const _PREHASH_ScriptMailRegistration = gMessageStringTable.getString("ScriptMailRegistration");
char const* const _PREHASH_AgentSetAppearance = gMessageStringTable.getString("AgentSetAppearance");
char const* const _PREHASH_AvatarAppearance = gMessageStringTable.getString("AvatarAppearance");
char const* const _PREHASH_RegionData = gMessageStringTable.getString("RegionData");
char const* const _PREHASH_RequestingRegionData = gMessageStringTable.getString("RequestingRegionData");
char const* const _PREHASH_LandingRegionData = gMessageStringTable.getString("LandingRegionData");
char const* const _PREHASH_SitTransform = gMessageStringTable.getString("SitTransform");
char const* const _PREHASH_TerrainBase0 = gMessageStringTable.getString("TerrainBase0");
char const* const _PREHASH_SkillsMask = gMessageStringTable.getString("SkillsMask");
char const* const _PREHASH_AtAxis = gMessageStringTable.getString("AtAxis");
char const* const _PREHASH_TerrainBase1 = gMessageStringTable.getString("TerrainBase1");
char const* const _PREHASH_Reason = gMessageStringTable.getString("Reason");
char const* const _PREHASH_TerrainBase2 = gMessageStringTable.getString("TerrainBase2");
char const* const _PREHASH_TerrainBase3 = gMessageStringTable.getString("TerrainBase3");
char const* const _PREHASH_Params = gMessageStringTable.getString("Params");
char const* const _PREHASH_PingID = gMessageStringTable.getString("PingID");
char const* const _PREHASH_Change = gMessageStringTable.getString("Change");
char const* const _PREHASH_Height = gMessageStringTable.getString("Height");
char const* const _PREHASH_Region = gMessageStringTable.getString("Region");
char const* const _PREHASH_TelehubInfo = gMessageStringTable.getString("TelehubInfo");
char const* const _PREHASH_StateSave = gMessageStringTable.getString("StateSave");
char const* const _PREHASH_RoleData = gMessageStringTable.getString("RoleData");
char const* const _PREHASH_AgentAnimation = gMessageStringTable.getString("AgentAnimation");
char const* const _PREHASH_AvatarAnimation = gMessageStringTable.getString("AvatarAnimation");
char const* const _PREHASH_ObjectAnimation = gMessageStringTable.getString("ObjectAnimation");
char const* const _PREHASH_LogDwellTime = gMessageStringTable.getString("LogDwellTime");
char const* const _PREHASH_ParcelGodMarkAsContent = gMessageStringTable.getString("ParcelGodMarkAsContent");
char const* const _PREHASH_UsePhysics = gMessageStringTable.getString("UsePhysics");
char const* const _PREHASH_RegionDenyTransacted = gMessageStringTable.getString("RegionDenyTransacted");
char const* const _PREHASH_JointType = gMessageStringTable.getString("JointType");
char const* const _PREHASH_ObjectTaxEstimate = gMessageStringTable.getString("ObjectTaxEstimate");
char const* const _PREHASH_LightTaxEstimate = gMessageStringTable.getString("LightTaxEstimate");
char const* const _PREHASH_LandTaxEstimate = gMessageStringTable.getString("LandTaxEstimate");
char const* const _PREHASH_TeleportLandingStatusChanged = gMessageStringTable.getString("TeleportLandingStatusChanged");
char const* const _PREHASH_GroupTaxEstimate = gMessageStringTable.getString("GroupTaxEstimate");
char const* const _PREHASH_AvgViewerFPS = gMessageStringTable.getString("AvgViewerFPS");
char const* const _PREHASH_Buttons = gMessageStringTable.getString("Buttons");
char const* const _PREHASH_Sender = gMessageStringTable.getString("Sender");
char const* const _PREHASH_Dialog = gMessageStringTable.getString("Dialog");
char const* const _PREHASH_TargetData = gMessageStringTable.getString("TargetData");
char const* const _PREHASH_DestID = gMessageStringTable.getString("DestID");
char const* const _PREHASH_ObjectDelete = gMessageStringTable.getString("ObjectDelete");
char const* const _PREHASH_Delete = gMessageStringTable.getString("Delete");
char const* const _PREHASH_EventGodDelete = gMessageStringTable.getString("EventGodDelete");
char const* const _PREHASH_LastTaxDate = gMessageStringTable.getString("LastTaxDate");
char const* const _PREHASH_MapImageID = gMessageStringTable.getString("MapImageID");
char const* const _PREHASH_EndDateTime = gMessageStringTable.getString("EndDateTime");
char const* const _PREHASH_TerrainDetail0 = gMessageStringTable.getString("TerrainDetail0");
char const* const _PREHASH_TerrainDetail1 = gMessageStringTable.getString("TerrainDetail1");
char const* const _PREHASH_TerrainDetail2 = gMessageStringTable.getString("TerrainDetail2");
char const* const _PREHASH_TerrainDetail3 = gMessageStringTable.getString("TerrainDetail3");
char const* const _PREHASH_Offset = gMessageStringTable.getString("Offset");
char const* const _PREHASH_ObjectDelink = gMessageStringTable.getString("ObjectDelink");
char const* const _PREHASH_TargetObject = gMessageStringTable.getString("TargetObject");
char const* const _PREHASH_IsEstateManager = gMessageStringTable.getString("IsEstateManager");
char const* const _PREHASH_CancelAuction = gMessageStringTable.getString("CancelAuction");
char const* const _PREHASH_ObjectDetach = gMessageStringTable.getString("ObjectDetach");
char const* const _PREHASH_Compressed = gMessageStringTable.getString("Compressed");
char const* const _PREHASH_PathBegin = gMessageStringTable.getString("PathBegin");
char const* const _PREHASH_BypassRaycast = gMessageStringTable.getString("BypassRaycast");
char const* const _PREHASH_WinnerID = gMessageStringTable.getString("WinnerID");
char const* const _PREHASH_ChannelType = gMessageStringTable.getString("ChannelType");
char const* const _PREHASH_NonExemptMembers = gMessageStringTable.getString("NonExemptMembers");
char const* const _PREHASH_Agents = gMessageStringTable.getString("Agents");
char const* const _PREHASH_MemberData = gMessageStringTable.getString("MemberData");
char const* const _PREHASH_ToGroupID = gMessageStringTable.getString("ToGroupID");
char const* const _PREHASH_ImageNotInDatabase = gMessageStringTable.getString("ImageNotInDatabase");
char const* const _PREHASH_StartDate = gMessageStringTable.getString("StartDate");
char const* const _PREHASH_AnimID = gMessageStringTable.getString("AnimID");
char const* const _PREHASH_Serial = gMessageStringTable.getString("Serial");
char const* const _PREHASH_AbuseRegionName = gMessageStringTable.getString("AbuseRegionName");
char const* const _PREHASH_ModifyLand = gMessageStringTable.getString("ModifyLand");
char const* const _PREHASH_Digest = gMessageStringTable.getString("Digest");
char const* const _PREHASH_Victim = gMessageStringTable.getString("Victim");
char const* const _PREHASH_Script = gMessageStringTable.getString("Script");
char const* const _PREHASH_PickInfoReply = gMessageStringTable.getString("PickInfoReply");
char const* const _PREHASH_MoneyBalanceReply = gMessageStringTable.getString("MoneyBalanceReply");
char const* const _PREHASH_RoutedMoneyBalanceReply = gMessageStringTable.getString("RoutedMoneyBalanceReply");
char const* const _PREHASH_RoleID = gMessageStringTable.getString("RoleID");
char const* const _PREHASH_RegionInfo = gMessageStringTable.getString("RegionInfo");
char const* const _PREHASH_GodUpdateRegionInfo = gMessageStringTable.getString("GodUpdateRegionInfo");
char const* const _PREHASH_StartAnim = gMessageStringTable.getString("StartAnim");
char const* const _PREHASH_Action = gMessageStringTable.getString("Action");
char const* const _PREHASH_Location = gMessageStringTable.getString("Location");
char const* const _PREHASH_Rights = gMessageStringTable.getString("Rights");
char const* const _PREHASH_SearchDir = gMessageStringTable.getString("SearchDir");
char const* const _PREHASH_TransferRequest = gMessageStringTable.getString("TransferRequest");
char const* const _PREHASH_ScriptSensorRequest = gMessageStringTable.getString("ScriptSensorRequest");
char const* const _PREHASH_MoneyTransferRequest = gMessageStringTable.getString("MoneyTransferRequest");
char const* const _PREHASH_EjectGroupMemberRequest = gMessageStringTable.getString("EjectGroupMemberRequest");
char const* const _PREHASH_SkillsText = gMessageStringTable.getString("SkillsText");
char const* const _PREHASH_Resent = gMessageStringTable.getString("Resent");
char const* const _PREHASH_Center = gMessageStringTable.getString("Center");
char const* const _PREHASH_SharedData = gMessageStringTable.getString("SharedData");
char const* const _PREHASH_PSBlock = gMessageStringTable.getString("PSBlock");
char const* const _PREHASH_UUIDNameBlock = gMessageStringTable.getString("UUIDNameBlock");
char const* const _PREHASH_GroupTitleUpdate = gMessageStringTable.getString("GroupTitleUpdate");
char const* const _PREHASH_Method = gMessageStringTable.getString("Method");
char const* const _PREHASH_TouchName = gMessageStringTable.getString("TouchName");
char const* const _PREHASH_UpdateType = gMessageStringTable.getString("UpdateType");
char const* const _PREHASH_KickedFromEstateID = gMessageStringTable.getString("KickedFromEstateID");
char const* const _PREHASH_CandidateID = gMessageStringTable.getString("CandidateID");
char const* const _PREHASH_ParamData = gMessageStringTable.getString("ParamData");
char const* const _PREHASH_GodlikeMessage = gMessageStringTable.getString("GodlikeMessage");
char const* const _PREHASH_SystemMessage = gMessageStringTable.getString("SystemMessage");
char const* const _PREHASH_BodyRotation = gMessageStringTable.getString("BodyRotation");
char const* const _PREHASH_SearchRegions = gMessageStringTable.getString("SearchRegions");
char const* const _PREHASH_AnimationData = gMessageStringTable.getString("AnimationData");
char const* const _PREHASH_StatID = gMessageStringTable.getString("StatID");
char const* const _PREHASH_ItemID = gMessageStringTable.getString("ItemID");
char const* const _PREHASH_ScriptDialogReply = gMessageStringTable.getString("ScriptDialogReply");
char const* const _PREHASH_RegionIDAndHandleReply = gMessageStringTable.getString("RegionIDAndHandleReply");
char const* const _PREHASH_CameraAtOffset = gMessageStringTable.getString("CameraAtOffset");
char const* const _PREHASH_VoteID = gMessageStringTable.getString("VoteID");
char const* const _PREHASH_ParcelGodForceOwner = gMessageStringTable.getString("ParcelGodForceOwner");
char const* const _PREHASH_Filter = gMessageStringTable.getString("Filter");
char const* const _PREHASH_InviteData = gMessageStringTable.getString("InviteData");
char const* const _PREHASH_PCode = gMessageStringTable.getString("PCode");
char const* const _PREHASH_SearchPos = gMessageStringTable.getString("SearchPos");
char const* const _PREHASH_PreyID = gMessageStringTable.getString("PreyID");
char const* const _PREHASH_TerrainLowerLimit = gMessageStringTable.getString("TerrainLowerLimit");
char const* const _PREHASH_EventFlags = gMessageStringTable.getString("EventFlags");
char const* const _PREHASH_TallyVotes = gMessageStringTable.getString("TallyVotes");
char const* const _PREHASH_Result = gMessageStringTable.getString("Result");
char const* const _PREHASH_LookAt = gMessageStringTable.getString("LookAt");
char const* const _PREHASH_SearchOrder = gMessageStringTable.getString("SearchOrder");
char const* const _PREHASH_PayButton = gMessageStringTable.getString("PayButton");
char const* const _PREHASH_SelfCount = gMessageStringTable.getString("SelfCount");
char const* const _PREHASH_PacketCount = gMessageStringTable.getString("PacketCount");
char const* const _PREHASH_ParcelBuyPass = gMessageStringTable.getString("ParcelBuyPass");
char const* const _PREHASH_OldItemID = gMessageStringTable.getString("OldItemID");
char const* const _PREHASH_RegionPort = gMessageStringTable.getString("RegionPort");
char const* const _PREHASH_Bitmap = gMessageStringTable.getString("Bitmap");
char const* const _PREHASH_CacheMissType = gMessageStringTable.getString("CacheMissType");
char const* const _PREHASH_VFileID = gMessageStringTable.getString("VFileID");
char const* const _PREHASH_GroupInsigniaID = gMessageStringTable.getString("GroupInsigniaID");
char const* const _PREHASH_Online = gMessageStringTable.getString("Online");
char const* const _PREHASH_KickFlags = gMessageStringTable.getString("KickFlags");
char const* const _PREHASH_CovenantID = gMessageStringTable.getString("CovenantID");
char const* const _PREHASH_SysCPU = gMessageStringTable.getString("SysCPU");
char const* const _PREHASH_EMail = gMessageStringTable.getString("EMail");
char const* const _PREHASH_AggregatePermTextures = gMessageStringTable.getString("AggregatePermTextures");
char const* const _PREHASH_ChatChannel = gMessageStringTable.getString("ChatChannel");
char const* const _PREHASH_OwnerData = gMessageStringTable.getString("OwnerData");
char const* const _PREHASH_ReturnID = gMessageStringTable.getString("ReturnID");
char const* const _PREHASH_ObjectAttach = gMessageStringTable.getString("ObjectAttach");
char const* const _PREHASH_TargetPort = gMessageStringTable.getString("TargetPort");
char const* const _PREHASH_ObjectSpinStop = gMessageStringTable.getString("ObjectSpinStop");
char const* const _PREHASH_FullID = gMessageStringTable.getString("FullID");
char const* const _PREHASH_ActivateGroup = gMessageStringTable.getString("ActivateGroup");
char const* const _PREHASH_SysGPU = gMessageStringTable.getString("SysGPU");
char const* const _PREHASH_AvatarInterestsReply = gMessageStringTable.getString("AvatarInterestsReply");
char const* const _PREHASH_StartLure = gMessageStringTable.getString("StartLure");
char const* const _PREHASH_SysRAM = gMessageStringTable.getString("SysRAM");
char const* const _PREHASH_SitPosition = gMessageStringTable.getString("SitPosition");
char const* const _PREHASH_StartTime = gMessageStringTable.getString("StartTime");
char const* const _PREHASH_BornOn = gMessageStringTable.getString("BornOn");
char const* const _PREHASH_CameraCollidePlane = gMessageStringTable.getString("CameraCollidePlane");
char const* const _PREHASH_EconomyDataRequest = gMessageStringTable.getString("EconomyDataRequest");
char const* const _PREHASH_TeleportLureRequest = gMessageStringTable.getString("TeleportLureRequest");
char const* const _PREHASH_FolderID = gMessageStringTable.getString("FolderID");
char const* const _PREHASH_RegionHandleRequest = gMessageStringTable.getString("RegionHandleRequest");
char const* const _PREHASH_ScriptDataRequest = gMessageStringTable.getString("ScriptDataRequest");
char const* const _PREHASH_GroupRoleDataRequest = gMessageStringTable.getString("GroupRoleDataRequest");
char const* const _PREHASH_GroupTitlesRequest = gMessageStringTable.getString("GroupTitlesRequest");
char const* const _PREHASH_AgentWearablesRequest = gMessageStringTable.getString("AgentWearablesRequest");
char const* const _PREHASH_MapBlockRequest = gMessageStringTable.getString("MapBlockRequest");
char const* const _PREHASH_LureID = gMessageStringTable.getString("LureID");
char const* const _PREHASH_CopyCenters = gMessageStringTable.getString("CopyCenters");
char const* const _PREHASH_ParamList = gMessageStringTable.getString("ParamList");
char const* const _PREHASH_InventorySerial = gMessageStringTable.getString("InventorySerial");
char const* const _PREHASH_EdgeDataPacket = gMessageStringTable.getString("EdgeDataPacket");
char const* const _PREHASH_AvatarPickerReply = gMessageStringTable.getString("AvatarPickerReply");
char const* const _PREHASH_ParcelDwellReply = gMessageStringTable.getString("ParcelDwellReply");
char const* const _PREHASH_IsForSale = gMessageStringTable.getString("IsForSale");
char const* const _PREHASH_MuteID = gMessageStringTable.getString("MuteID");
char const* const _PREHASH_MeanCollisionAlert = gMessageStringTable.getString("MeanCollisionAlert");
char const* const _PREHASH_CanAcceptTasks = gMessageStringTable.getString("CanAcceptTasks");
char const* const _PREHASH_ItemData = gMessageStringTable.getString("ItemData");
char const* const _PREHASH_AnimationList = gMessageStringTable.getString("AnimationList");
char const* const _PREHASH_Reputation = gMessageStringTable.getString("Reputation");
char const* const _PREHASH_IntValue = gMessageStringTable.getString("IntValue");
char const* const _PREHASH_TargetType = gMessageStringTable.getString("TargetType");
char const* const _PREHASH_Amount = gMessageStringTable.getString("Amount");
char const* const _PREHASH_HasAttachment = gMessageStringTable.getString("HasAttachment");
char const* const _PREHASH_UpdateAttachment = gMessageStringTable.getString("UpdateAttachment");
char const* const _PREHASH_RemoveAttachment = gMessageStringTable.getString("RemoveAttachment");
char const* const _PREHASH_HeightWidthBlock = gMessageStringTable.getString("HeightWidthBlock");
char const* const _PREHASH_RequestObjectPropertiesFamily = gMessageStringTable.getString("RequestObjectPropertiesFamily");
char const* const _PREHASH_ObjectPropertiesFamily = gMessageStringTable.getString("ObjectPropertiesFamily");
char const* const _PREHASH_UserData = gMessageStringTable.getString("UserData");
char const* const _PREHASH_IsReadable = gMessageStringTable.getString("IsReadable");
char const* const _PREHASH_PathCurve = gMessageStringTable.getString("PathCurve");
char const* const _PREHASH_Status = gMessageStringTable.getString("Status");
char const* const _PREHASH_FromGroup = gMessageStringTable.getString("FromGroup");
char const* const _PREHASH_AlreadyVoted = gMessageStringTable.getString("AlreadyVoted");
char const* const _PREHASH_PlacesReply = gMessageStringTable.getString("PlacesReply");
char const* const _PREHASH_DirPlacesReply = gMessageStringTable.getString("DirPlacesReply");
char const* const _PREHASH_ParcelBuy = gMessageStringTable.getString("ParcelBuy");
char const* const _PREHASH_DirFindQueryBackend = gMessageStringTable.getString("DirFindQueryBackend");
char const* const _PREHASH_DirPlacesQueryBackend = gMessageStringTable.getString("DirPlacesQueryBackend");
char const* const _PREHASH_DirClassifiedQueryBackend = gMessageStringTable.getString("DirClassifiedQueryBackend");
char const* const _PREHASH_DirLandQueryBackend = gMessageStringTable.getString("DirLandQueryBackend");
char const* const _PREHASH_DirPopularQueryBackend = gMessageStringTable.getString("DirPopularQueryBackend");
char const* const _PREHASH_HistoryData = gMessageStringTable.getString("HistoryData");
char const* const _PREHASH_SnapshotID = gMessageStringTable.getString("SnapshotID");
char const* const _PREHASH_Aspect = gMessageStringTable.getString("Aspect");
char const* const _PREHASH_ParamSize = gMessageStringTable.getString("ParamSize");
char const* const _PREHASH_VoteCast = gMessageStringTable.getString("VoteCast");
char const* const _PREHASH_EveryoneMask = gMessageStringTable.getString("EveryoneMask");
char const* const _PREHASH_ObjectSpinUpdate = gMessageStringTable.getString("ObjectSpinUpdate");
char const* const _PREHASH_MaturePublish = gMessageStringTable.getString("MaturePublish");
char const* const _PREHASH_UseExistingAsset = gMessageStringTable.getString("UseExistingAsset");
char const* const _PREHASH_Powers = gMessageStringTable.getString("Powers");
char const* const _PREHASH_ParcelLocalID = gMessageStringTable.getString("ParcelLocalID");
char const* const _PREHASH_TeleportCancel = gMessageStringTable.getString("TeleportCancel");
char const* const _PREHASH_UnixTime = gMessageStringTable.getString("UnixTime");
char const* const _PREHASH_QueryFlags = gMessageStringTable.getString("QueryFlags");
char const* const _PREHASH_AlwaysRun = gMessageStringTable.getString("AlwaysRun");
char const* const _PREHASH_Bottom = gMessageStringTable.getString("Bottom");
char const* const _PREHASH_ButtonData = gMessageStringTable.getString("ButtonData");
char const* const _PREHASH_SoundData = gMessageStringTable.getString("SoundData");
char const* const _PREHASH_ViewerStats = gMessageStringTable.getString("ViewerStats");
char const* const _PREHASH_RegionHandshake = gMessageStringTable.getString("RegionHandshake");
char const* const _PREHASH_ObjectDescription = gMessageStringTable.getString("ObjectDescription");
char const* const _PREHASH_Description = gMessageStringTable.getString("Description");
char const* const _PREHASH_ParamType = gMessageStringTable.getString("ParamType");
char const* const _PREHASH_UUIDNameReply = gMessageStringTable.getString("UUIDNameReply");
char const* const _PREHASH_UUIDGroupNameReply = gMessageStringTable.getString("UUIDGroupNameReply");
char const* const _PREHASH_SaveAssetIntoInventory = gMessageStringTable.getString("SaveAssetIntoInventory");
char const* const _PREHASH_UserInfo = gMessageStringTable.getString("UserInfo");
char const* const _PREHASH_AnimSequenceID = gMessageStringTable.getString("AnimSequenceID");
char const* const _PREHASH_NVPairs = gMessageStringTable.getString("NVPairs");
char const* const _PREHASH_GroupNoticesListRequest = gMessageStringTable.getString("GroupNoticesListRequest");
char const* const _PREHASH_ParcelAccessListRequest = gMessageStringTable.getString("ParcelAccessListRequest");
char const* const _PREHASH_MuteListRequest = gMessageStringTable.getString("MuteListRequest");
char const* const _PREHASH_RpcChannelRequest = gMessageStringTable.getString("RpcChannelRequest");
char const* const _PREHASH_LandStatRequest = gMessageStringTable.getString("LandStatRequest");
char const* const _PREHASH_PlacesQuery = gMessageStringTable.getString("PlacesQuery");
char const* const _PREHASH_DirPlacesQuery = gMessageStringTable.getString("DirPlacesQuery");
char const* const _PREHASH_SortOrder = gMessageStringTable.getString("SortOrder");
char const* const _PREHASH_Hunter = gMessageStringTable.getString("Hunter");
char const* const _PREHASH_SunAngVelocity = gMessageStringTable.getString("SunAngVelocity");
char const* const _PREHASH_BinaryBucket = gMessageStringTable.getString("BinaryBucket");
char const* const _PREHASH_ImagePacket = gMessageStringTable.getString("ImagePacket");
char const* const _PREHASH_StartGroupProposal = gMessageStringTable.getString("StartGroupProposal");
char const* const _PREHASH_EnergyLevel = gMessageStringTable.getString("EnergyLevel");
char const* const _PREHASH_PriceForListing = gMessageStringTable.getString("PriceForListing");
char const* const _PREHASH_Scale = gMessageStringTable.getString("Scale");
char const* const _PREHASH_EstateCovenantReply = gMessageStringTable.getString("EstateCovenantReply");
char const* const _PREHASH_ParentEstateID = gMessageStringTable.getString("ParentEstateID");
char const* const _PREHASH_Extra2 = gMessageStringTable.getString("Extra2");
char const* const _PREHASH_Throttle = gMessageStringTable.getString("Throttle");
char const* const _PREHASH_SimIP = gMessageStringTable.getString("SimIP");
char const* const _PREHASH_GodID = gMessageStringTable.getString("GodID");
char const* const _PREHASH_VoteItem = gMessageStringTable.getString("VoteItem");
char const* const _PREHASH_SitRotation = gMessageStringTable.getString("SitRotation");
char const* const _PREHASH_SnapSelection = gMessageStringTable.getString("SnapSelection");
char const* const _PREHASH_SoundTrigger = gMessageStringTable.getString("SoundTrigger");
char const* const _PREHASH_TerrainRaiseLimit = gMessageStringTable.getString("TerrainRaiseLimit");
char const* const _PREHASH_Quorum = gMessageStringTable.getString("Quorum");
char const* const _PREHASH_AgentBlock = gMessageStringTable.getString("AgentBlock");
char const* const _PREHASH_CommandBlock = gMessageStringTable.getString("CommandBlock");
char const* const _PREHASH_SpawnPointPos = gMessageStringTable.getString("SpawnPointPos");
char const* const _PREHASH_VolumeDetail = gMessageStringTable.getString("VolumeDetail");
char const* const _PREHASH_FromAgentName = gMessageStringTable.getString("FromAgentName");
char const* const _PREHASH_Range = gMessageStringTable.getString("Range");
char const* const _PREHASH_DirectoryVisibility = gMessageStringTable.getString("DirectoryVisibility");
char const* const _PREHASH_PublicIP = gMessageStringTable.getString("PublicIP");
char const* const _PREHASH_TeleportFailed = gMessageStringTable.getString("TeleportFailed");
char const* const _PREHASH_PreloadSound = gMessageStringTable.getString("PreloadSound");
char const* const _PREHASH_ScreenshotID = gMessageStringTable.getString("ScreenshotID");
char const* const _PREHASH_CovenantTimestamp = gMessageStringTable.getString("CovenantTimestamp");
char const* const _PREHASH_OldestUnacked = gMessageStringTable.getString("OldestUnacked");
char const* const _PREHASH_SimulatorIP = gMessageStringTable.getString("SimulatorIP");
char const* const _PREHASH_Value = gMessageStringTable.getString("Value");
char const* const _PREHASH_JointAxisOrAnchor = gMessageStringTable.getString("JointAxisOrAnchor");
char const* const _PREHASH_Test0 = gMessageStringTable.getString("Test0");
char const* const _PREHASH_Test1 = gMessageStringTable.getString("Test1");
char const* const _PREHASH_Test2 = gMessageStringTable.getString("Test2");
char const* const _PREHASH_SunPhase = gMessageStringTable.getString("SunPhase");
char const* const _PREHASH_ParcelDivide = gMessageStringTable.getString("ParcelDivide");
char const* const _PREHASH_Field = gMessageStringTable.getString("Field");
char const* const _PREHASH_Ratio = gMessageStringTable.getString("Ratio");
char const* const _PREHASH_JoinGroupReply = gMessageStringTable.getString("JoinGroupReply");
char const* const _PREHASH_LiveHelpGroupReply = gMessageStringTable.getString("LiveHelpGroupReply");
char const* const _PREHASH_Score = gMessageStringTable.getString("Score");
char const* const _PREHASH_Image = gMessageStringTable.getString("Image");
char const* const _PREHASH_ObjectClickAction = gMessageStringTable.getString("ObjectClickAction");
char const* const _PREHASH_Parameter = gMessageStringTable.getString("Parameter");
char const* const _PREHASH_Flags = gMessageStringTable.getString("Flags");
char const* const _PREHASH_Plane = gMessageStringTable.getString("Plane");
char const* const _PREHASH_Width = gMessageStringTable.getString("Width");
char const* const _PREHASH_Right = gMessageStringTable.getString("Right");
char const* const _PREHASH_DirFindQuery = gMessageStringTable.getString("DirFindQuery");
char const* const _PREHASH_Textures = gMessageStringTable.getString("Textures");
char const* const _PREHASH_EventData = gMessageStringTable.getString("EventData");
char const* const _PREHASH_Final = gMessageStringTable.getString("Final");
char const* const _PREHASH_System = gMessageStringTable.getString("System");
char const* const _PREHASH_TelehubPos = gMessageStringTable.getString("TelehubPos");
char const* const _PREHASH_ReportAutosaveCrash = gMessageStringTable.getString("ReportAutosaveCrash");
char const* const _PREHASH_CreateTrustedCircuit = gMessageStringTable.getString("CreateTrustedCircuit");
char const* const _PREHASH_DenyTrustedCircuit = gMessageStringTable.getString("DenyTrustedCircuit");
char const* const _PREHASH_RequestTrustedCircuit = gMessageStringTable.getString("RequestTrustedCircuit");
char const* const _PREHASH_Codec = gMessageStringTable.getString("Codec");
char const* const _PREHASH_Modal = gMessageStringTable.getString("Modal");
char const* const _PREHASH_ChildAgentUnknown = gMessageStringTable.getString("ChildAgentUnknown");
char const* const _PREHASH_LandingType = gMessageStringTable.getString("LandingType");
char const* const _PREHASH_ScriptRunningReply = gMessageStringTable.getString("ScriptRunningReply");
char const* const _PREHASH_Reply = gMessageStringTable.getString("Reply");
char const* const _PREHASH_GroupAccountDetailsReply = gMessageStringTable.getString("GroupAccountDetailsReply");
char const* const _PREHASH_TelehubRot = gMessageStringTable.getString("TelehubRot");
char const* const _PREHASH_AcceptFriendship = gMessageStringTable.getString("AcceptFriendship");
char const* const _PREHASH_ItemType = gMessageStringTable.getString("ItemType");
char const* const _PREHASH_DwellInfo = gMessageStringTable.getString("DwellInfo");
char const* const _PREHASH_AgentResume = gMessageStringTable.getString("AgentResume");
char const* const _PREHASH_MailFilter = gMessageStringTable.getString("MailFilter");
char const* const _PREHASH_Disconnect = gMessageStringTable.getString("Disconnect");
char const* const _PREHASH_SimPosition = gMessageStringTable.getString("SimPosition");
char const* const _PREHASH_SimWideTotalPrims = gMessageStringTable.getString("SimWideTotalPrims");
char const* const _PREHASH_Index = gMessageStringTable.getString("Index");
char const* const _PREHASH_SimFilename = gMessageStringTable.getString("SimFilename");
char const* const _PREHASH_LastOwnerID = gMessageStringTable.getString("LastOwnerID");
char const* const _PREHASH_GroupNoticeRequest = gMessageStringTable.getString("GroupNoticeRequest");
char const* const _PREHASH_EmailMessageRequest = gMessageStringTable.getString("EmailMessageRequest");
char const* const _PREHASH_MapItemRequest = gMessageStringTable.getString("MapItemRequest");
char const* const _PREHASH_AgentCount = gMessageStringTable.getString("AgentCount");
char const* const _PREHASH_MessageBlock = gMessageStringTable.getString("MessageBlock");
char const* const _PREHASH_FuseBlock = gMessageStringTable.getString("FuseBlock");
char const* const _PREHASH_AgentGroupData = gMessageStringTable.getString("AgentGroupData");
char const* const _PREHASH_ClassifiedInfoUpdate = gMessageStringTable.getString("ClassifiedInfoUpdate");
char const* const _PREHASH_RegionPos = gMessageStringTable.getString("RegionPos");
char const* const _PREHASH_ParcelMediaUpdate = gMessageStringTable.getString("ParcelMediaUpdate");
char const* const _PREHASH_NoticeID = gMessageStringTable.getString("NoticeID");
char const* const _PREHASH_GridX = gMessageStringTable.getString("GridX");
char const* const _PREHASH_GridY = gMessageStringTable.getString("GridY");
char const* const _PREHASH_Title = gMessageStringTable.getString("Title");
char const* const _PREHASH_AuctionID = gMessageStringTable.getString("AuctionID");
char const* const _PREHASH_VoteType = gMessageStringTable.getString("VoteType");
char const* const _PREHASH_CategoryID = gMessageStringTable.getString("CategoryID");
char const* const _PREHASH_Token = gMessageStringTable.getString("Token");
char const* const _PREHASH_AggregatePerms = gMessageStringTable.getString("AggregatePerms");
char const* const _PREHASH_ObjectSelect = gMessageStringTable.getString("ObjectSelect");
char const* const _PREHASH_ForceObjectSelect = gMessageStringTable.getString("ForceObjectSelect");
char const* const _PREHASH_Price = gMessageStringTable.getString("Price");
char const* const _PREHASH_SunDirection = gMessageStringTable.getString("SunDirection");
char const* const _PREHASH_FromName = gMessageStringTable.getString("FromName");
char const* const _PREHASH_ChangeInventoryItemFlags = gMessageStringTable.getString("ChangeInventoryItemFlags");
char const* const _PREHASH_Force = gMessageStringTable.getString("Force");
char const* const _PREHASH_TransactionBlock = gMessageStringTable.getString("TransactionBlock");
char const* const _PREHASH_PowersMask = gMessageStringTable.getString("PowersMask");
char const* const _PREHASH_Stamp = gMessageStringTable.getString("Stamp");
char const* const _PREHASH_TotalCredits = gMessageStringTable.getString("TotalCredits");
char const* const _PREHASH_State = gMessageStringTable.getString("State");
char const* const _PREHASH_TextureIndex = gMessageStringTable.getString("TextureIndex");
char const* const _PREHASH_InviteeID = gMessageStringTable.getString("InviteeID");
char const* const _PREHASH_ParcelReclaim = gMessageStringTable.getString("ParcelReclaim");
char const* const _PREHASH_Money = gMessageStringTable.getString("Money");
char const* const _PREHASH_PathTwist = gMessageStringTable.getString("PathTwist");
char const* const _PREHASH_AuthBuyerID = gMessageStringTable.getString("AuthBuyerID");
char const* const _PREHASH_Color = gMessageStringTable.getString("Color");
char const* const _PREHASH_SourceType = gMessageStringTable.getString("SourceType");
char const* const _PREHASH_World = gMessageStringTable.getString("World");
char const* const _PREHASH_QueryData = gMessageStringTable.getString("QueryData");
char const* const _PREHASH_Users = gMessageStringTable.getString("Users");
char const* const _PREHASH_SysOS = gMessageStringTable.getString("SysOS");
char const* const _PREHASH_Notes = gMessageStringTable.getString("Notes");
char const* const _PREHASH_AvatarID = gMessageStringTable.getString("AvatarID");
char const* const _PREHASH_FounderID = gMessageStringTable.getString("FounderID");
char const* const _PREHASH_EndPointID = gMessageStringTable.getString("EndPointID");
char const* const _PREHASH_LocationLookAt = gMessageStringTable.getString("LocationLookAt");
char const* const _PREHASH_Sound = gMessageStringTable.getString("Sound");
char const* const _PREHASH_Cover = gMessageStringTable.getString("Cover");
char const* const _PREHASH_TotalObjectCount = gMessageStringTable.getString("TotalObjectCount");
char const* const _PREHASH_TextureEntry = gMessageStringTable.getString("TextureEntry");
char const* const _PREHASH_SquareMetersCommitted = gMessageStringTable.getString("SquareMetersCommitted");
char const* const _PREHASH_ChannelID = gMessageStringTable.getString("ChannelID");
char const* const _PREHASH_Dwell = gMessageStringTable.getString("Dwell");
char const* const _PREHASH_AgentUpdate = gMessageStringTable.getString("AgentUpdate");
char const* const _PREHASH_PickGodDelete = gMessageStringTable.getString("PickGodDelete");
char const* const _PREHASH_HostName = gMessageStringTable.getString("HostName");
char const* const _PREHASH_PriceParcelClaim = gMessageStringTable.getString("PriceParcelClaim");
char const* const _PREHASH_ParcelClaim = gMessageStringTable.getString("ParcelClaim");
char const* const _PREHASH_AgentPowers = gMessageStringTable.getString("AgentPowers");
char const* const _PREHASH_ProfileHollow = gMessageStringTable.getString("ProfileHollow");
char const* const _PREHASH_GroupRoleChanges = gMessageStringTable.getString("GroupRoleChanges");
char const* const _PREHASH_Count = gMessageStringTable.getString("Count");
char const* const _PREHASH_ObjectUpdateCompressed = gMessageStringTable.getString("ObjectUpdateCompressed");
char const* const _PREHASH_MuteFlags = gMessageStringTable.getString("MuteFlags");
char const* const _PREHASH_Group = gMessageStringTable.getString("Group");
char const* const _PREHASH_AgentPause = gMessageStringTable.getString("AgentPause");
char const* const _PREHASH_LanguagesText = gMessageStringTable.getString("LanguagesText");
char const* const _PREHASH_Error = gMessageStringTable.getString("Error");
char const* const _PREHASH_InternalScriptMail = gMessageStringTable.getString("InternalScriptMail");
char const* const _PREHASH_FindAgent = gMessageStringTable.getString("FindAgent");
char const* const _PREHASH_AgentData = gMessageStringTable.getString("AgentData");
char const* const _PREHASH_FolderData = gMessageStringTable.getString("FolderData");
char const* const _PREHASH_AssetBlock = gMessageStringTable.getString("AssetBlock");
char const* const _PREHASH_AcceptNotices = gMessageStringTable.getString("AcceptNotices");
char const* const _PREHASH_SetGroupAcceptNotices = gMessageStringTable.getString("SetGroupAcceptNotices");
char const* const _PREHASH_CloseCircuit = gMessageStringTable.getString("CloseCircuit");
char const* const _PREHASH_TeleportFinish = gMessageStringTable.getString("TeleportFinish");
char const* const _PREHASH_PathRevolutions = gMessageStringTable.getString("PathRevolutions");
char const* const _PREHASH_ClassifiedInfoReply = gMessageStringTable.getString("ClassifiedInfoReply");
char const* const _PREHASH_ParcelInfoReply = gMessageStringTable.getString("ParcelInfoReply");
char const* const _PREHASH_AutosaveData = gMessageStringTable.getString("AutosaveData");
char const* const _PREHASH_SetStartLocation = gMessageStringTable.getString("SetStartLocation");
char const* const _PREHASH_PassHours = gMessageStringTable.getString("PassHours");
char const* const _PREHASH_AttachmentPt = gMessageStringTable.getString("AttachmentPt");
char const* const _PREHASH_ParcelFlags = gMessageStringTable.getString("ParcelFlags");
char const* const _PREHASH_NumVotes = gMessageStringTable.getString("NumVotes");
char const* const _PREHASH_AvatarPickerRequest = gMessageStringTable.getString("AvatarPickerRequest");
char const* const _PREHASH_TeleportLocationRequest = gMessageStringTable.getString("TeleportLocationRequest");
char const* const _PREHASH_DataHomeLocationRequest = gMessageStringTable.getString("DataHomeLocationRequest");
char const* const _PREHASH_EventNotificationAddRequest = gMessageStringTable.getString("EventNotificationAddRequest");
char const* const _PREHASH_ParcelDwellRequest = gMessageStringTable.getString("ParcelDwellRequest");
char const* const _PREHASH_EventLocationRequest = gMessageStringTable.getString("EventLocationRequest");
char const* const _PREHASH_SetStartLocationRequest = gMessageStringTable.getString("SetStartLocationRequest");
char const* const _PREHASH_QueryStart = gMessageStringTable.getString("QueryStart");
char const* const _PREHASH_EjectData = gMessageStringTable.getString("EjectData");
char const* const _PREHASH_AvatarTextureUpdate = gMessageStringTable.getString("AvatarTextureUpdate");
char const* const _PREHASH_RPCServerPort = gMessageStringTable.getString("RPCServerPort");
char const* const _PREHASH_Bytes = gMessageStringTable.getString("Bytes");
char const* const _PREHASH_Extra = gMessageStringTable.getString("Extra");
char const* const _PREHASH_ForceScriptControlRelease = gMessageStringTable.getString("ForceScriptControlRelease");
char const* const _PREHASH_ParcelRelease = gMessageStringTable.getString("ParcelRelease");
char const* const _PREHASH_VFileType = gMessageStringTable.getString("VFileType");
char const* const _PREHASH_EjectGroupMemberReply = gMessageStringTable.getString("EjectGroupMemberReply");
char const* const _PREHASH_ImageData = gMessageStringTable.getString("ImageData");
char const* const _PREHASH_SimulatorViewerTimeMessage = gMessageStringTable.getString("SimulatorViewerTimeMessage");
char const* const _PREHASH_Rotation = gMessageStringTable.getString("Rotation");
char const* const _PREHASH_Selection = gMessageStringTable.getString("Selection");
char const* const _PREHASH_TransactionData = gMessageStringTable.getString("TransactionData");
char const* const _PREHASH_OperationData = gMessageStringTable.getString("OperationData");
char const* const _PREHASH_ExpirationDate = gMessageStringTable.getString("ExpirationDate");
char const* const _PREHASH_ParcelDeedToGroup = gMessageStringTable.getString("ParcelDeedToGroup");
char const* const _PREHASH_AvatarPicksReply = gMessageStringTable.getString("AvatarPicksReply");
char const* const _PREHASH_GroupTitlesReply = gMessageStringTable.getString("GroupTitlesReply");
char const* const _PREHASH_AgentInfo = gMessageStringTable.getString("AgentInfo");
char const* const _PREHASH_MoneyTransferBackend = gMessageStringTable.getString("MoneyTransferBackend");
char const* const _PREHASH_NextOwnerMask = gMessageStringTable.getString("NextOwnerMask");
char const* const _PREHASH_MuteData = gMessageStringTable.getString("MuteData");
char const* const _PREHASH_PassPrice = gMessageStringTable.getString("PassPrice");
char const* const _PREHASH_SourceID = gMessageStringTable.getString("SourceID");
char const* const _PREHASH_IsDestGroup = gMessageStringTable.getString("IsDestGroup");
char const* const _PREHASH_IsSourceGroup = gMessageStringTable.getString("IsSourceGroup");
char const* const _PREHASH_ItemDescription = gMessageStringTable.getString("ItemDescription");
char const* const _PREHASH_ChangeUserRights = gMessageStringTable.getString("ChangeUserRights");
char const* const _PREHASH_TeleportFlags = gMessageStringTable.getString("TeleportFlags");
char const* const _PREHASH_SlaveParcelData = gMessageStringTable.getString("SlaveParcelData");
char const* const _PREHASH_AssetData = gMessageStringTable.getString("AssetData");
char const* const _PREHASH_MultipleObjectUpdate = gMessageStringTable.getString("MultipleObjectUpdate");
char const* const _PREHASH_ObjectUpdate = gMessageStringTable.getString("ObjectUpdate");
char const* const _PREHASH_ImprovedTerseObjectUpdate = gMessageStringTable.getString("ImprovedTerseObjectUpdate");
char const* const _PREHASH_ConfirmXferPacket = gMessageStringTable.getString("ConfirmXferPacket");
char const* const _PREHASH_StartPingCheck = gMessageStringTable.getString("StartPingCheck");
char const* const _PREHASH_SimWideDeletes = gMessageStringTable.getString("SimWideDeletes");
char const* const _PREHASH_LandStatReply = gMessageStringTable.getString("LandStatReply");
char const* const _PREHASH_AgentList = gMessageStringTable.getString("AgentList");
char const* const _PREHASH_SimApproved = gMessageStringTable.getString("SimApproved");
char const* const _PREHASH_RezObject = gMessageStringTable.getString("RezObject");
char const* const _PREHASH_TaskLocalID = gMessageStringTable.getString("TaskLocalID");
char const* const _PREHASH_ClaimDate = gMessageStringTable.getString("ClaimDate");
char const* const _PREHASH_MergeParcel = gMessageStringTable.getString("MergeParcel");
char const* const _PREHASH_Priority = gMessageStringTable.getString("Priority");
char const* const _PREHASH_QueryText = gMessageStringTable.getString("QueryText");
char const* const _PREHASH_GroupNoticeAdd = gMessageStringTable.getString("GroupNoticeAdd");
char const* const _PREHASH_ReturnType = gMessageStringTable.getString("ReturnType");
char const* const _PREHASH_FetchFolders = gMessageStringTable.getString("FetchFolders");
char const* const _PREHASH_SimulatorPublicHostBlock = gMessageStringTable.getString("SimulatorPublicHostBlock");
char const* const _PREHASH_HeaderData = gMessageStringTable.getString("HeaderData");
char const* const _PREHASH_RequestMultipleObjects = gMessageStringTable.getString("RequestMultipleObjects");
char const* const _PREHASH_RetrieveInstantMessages = gMessageStringTable.getString("RetrieveInstantMessages");
char const* const _PREHASH_OpenCircuit = gMessageStringTable.getString("OpenCircuit");
char const* const _PREHASH_CrossedRegion = gMessageStringTable.getString("CrossedRegion");
char const* const _PREHASH_DirGroupsReply = gMessageStringTable.getString("DirGroupsReply");
char const* const _PREHASH_AvatarGroupsReply = gMessageStringTable.getString("AvatarGroupsReply");
char const* const _PREHASH_EmailMessageReply = gMessageStringTable.getString("EmailMessageReply");
char const* const _PREHASH_GroupVoteHistoryItemReply = gMessageStringTable.getString("GroupVoteHistoryItemReply");
char const* const _PREHASH_ViewerPosition = gMessageStringTable.getString("ViewerPosition");
char const* const _PREHASH_Position = gMessageStringTable.getString("Position");
char const* const _PREHASH_ParentEstate = gMessageStringTable.getString("ParentEstate");
char const* const _PREHASH_EstateName = gMessageStringTable.getString("EstateName");
char const* const _PREHASH_MuteName = gMessageStringTable.getString("MuteName");
char const* const _PREHASH_ParcelRename = gMessageStringTable.getString("ParcelRename");
char const* const _PREHASH_ViewerFilename = gMessageStringTable.getString("ViewerFilename");
char const* const _PREHASH_UserReportInternal = gMessageStringTable.getString("UserReportInternal");
char const* const _PREHASH_AvatarPropertiesRequest = gMessageStringTable.getString("AvatarPropertiesRequest");
char const* const _PREHASH_ParcelPropertiesRequest = gMessageStringTable.getString("ParcelPropertiesRequest");
char const* const _PREHASH_GroupProfileRequest = gMessageStringTable.getString("GroupProfileRequest");
char const* const _PREHASH_AgentDataUpdateRequest = gMessageStringTable.getString("AgentDataUpdateRequest");
char const* const _PREHASH_PriceObjectScaleFactor = gMessageStringTable.getString("PriceObjectScaleFactor");
char const* const _PREHASH_OpenEnrollment = gMessageStringTable.getString("OpenEnrollment");
char const* const _PREHASH_GroupData = gMessageStringTable.getString("GroupData");
char const* const _PREHASH_RequestGodlikePowers = gMessageStringTable.getString("RequestGodlikePowers");
char const* const _PREHASH_GrantGodlikePowers = gMessageStringTable.getString("GrantGodlikePowers");
char const* const _PREHASH_TransactionID = gMessageStringTable.getString("TransactionID");
char const* const _PREHASH_DestinationID = gMessageStringTable.getString("DestinationID");
char const* const _PREHASH_Controls = gMessageStringTable.getString("Controls");
char const* const _PREHASH_FirstDetachAll = gMessageStringTable.getString("FirstDetachAll");
char const* const _PREHASH_EstateID = gMessageStringTable.getString("EstateID");
char const* const _PREHASH_ImprovedInstantMessage = gMessageStringTable.getString("ImprovedInstantMessage");
char const* const _PREHASH_CheckParcelSales = gMessageStringTable.getString("CheckParcelSales");
char const* const _PREHASH_ParcelSales = gMessageStringTable.getString("ParcelSales");
char const* const _PREHASH_CurrentInterval = gMessageStringTable.getString("CurrentInterval");
char const* const _PREHASH_NeighborBlock = gMessageStringTable.getString("NeighborBlock");
char const* const _PREHASH_LayerData = gMessageStringTable.getString("LayerData");
char const* const _PREHASH_NVPairData = gMessageStringTable.getString("NVPairData");
char const* const _PREHASH_TeleportLocal = gMessageStringTable.getString("TeleportLocal");
char const* const _PREHASH_EjecteeID = gMessageStringTable.getString("EjecteeID");
char const* const _PREHASH_VoteInitiator = gMessageStringTable.getString("VoteInitiator");
char const* const _PREHASH_TypeData = gMessageStringTable.getString("TypeData");
char const* const _PREHASH_OwnerIDs = gMessageStringTable.getString("OwnerIDs");
char const* const _PREHASH_SystemKickUser = gMessageStringTable.getString("SystemKickUser");
char const* const _PREHASH_TransactionTime = gMessageStringTable.getString("TransactionTime");
char const* const _PREHASH_TimeToLive = gMessageStringTable.getString("TimeToLive");
char const* const _PREHASH_OldAgentID = gMessageStringTable.getString("OldAgentID");
char const* const _PREHASH_MusicURL = gMessageStringTable.getString("MusicURL");
char const* const _PREHASH_ParcelPrimBonus = gMessageStringTable.getString("ParcelPrimBonus");
char const* const _PREHASH_EjectUser = gMessageStringTable.getString("EjectUser");
char const* const _PREHASH_CoarseLocationUpdate = gMessageStringTable.getString("CoarseLocationUpdate");
char const* const _PREHASH_ChildAgentPositionUpdate = gMessageStringTable.getString("ChildAgentPositionUpdate");
char const* const _PREHASH_StoreLocal = gMessageStringTable.getString("StoreLocal");
char const* const _PREHASH_GroupName = gMessageStringTable.getString("GroupName");
char const* const _PREHASH_PriceParcelRent = gMessageStringTable.getString("PriceParcelRent");
char const* const _PREHASH_SimStatus = gMessageStringTable.getString("SimStatus");
char const* const _PREHASH_TransactionSuccess = gMessageStringTable.getString("TransactionSuccess");
char const* const _PREHASH_LureType = gMessageStringTable.getString("LureType");
char const* const _PREHASH_GroupMask = gMessageStringTable.getString("GroupMask");
char const* const _PREHASH_SitObject = gMessageStringTable.getString("SitObject");
char const* const _PREHASH_Override = gMessageStringTable.getString("Override");
char const* const _PREHASH_LocomotionState = gMessageStringTable.getString("LocomotionState");
char const* const _PREHASH_PriceUpload = gMessageStringTable.getString("PriceUpload");
char const* const _PREHASH_RemoveParcel = gMessageStringTable.getString("RemoveParcel");
char const* const _PREHASH_ConfirmAuctionStart = gMessageStringTable.getString("ConfirmAuctionStart");
char const* const _PREHASH_RpcScriptRequestInbound = gMessageStringTable.getString("RpcScriptRequestInbound");
char const* const _PREHASH_ActiveGroupID = gMessageStringTable.getString("ActiveGroupID");
char const* const _PREHASH_ParcelReturnObjects = gMessageStringTable.getString("ParcelReturnObjects");
char const* const _PREHASH_TotalObjects = gMessageStringTable.getString("TotalObjects");
char const* const _PREHASH_ObjectExtraParams = gMessageStringTable.getString("ObjectExtraParams");
char const* const _PREHASH_Questions = gMessageStringTable.getString("Questions");
char const* const _PREHASH_TransferAbort = gMessageStringTable.getString("TransferAbort");
char const* const _PREHASH_TransferInventory = gMessageStringTable.getString("TransferInventory");
char const* const _PREHASH_RayTargetID = gMessageStringTable.getString("RayTargetID");
char const* const _PREHASH_ClaimPrice = gMessageStringTable.getString("ClaimPrice");
char const* const _PREHASH_ObjectProperties = gMessageStringTable.getString("ObjectProperties");
char const* const _PREHASH_ParcelProperties = gMessageStringTable.getString("ParcelProperties");
char const* const _PREHASH_EstateOwnerID = gMessageStringTable.getString("EstateOwnerID");
char const* const _PREHASH_LogoutRequest = gMessageStringTable.getString("LogoutRequest");
char const* const _PREHASH_AssetUploadRequest = gMessageStringTable.getString("AssetUploadRequest");
char const* const _PREHASH_TransactionInfo = gMessageStringTable.getString("TransactionInfo");
char const* const _PREHASH_TransactionType = gMessageStringTable.getString("TransactionType");
char const* const _PREHASH_AvatarPropertiesUpdate = gMessageStringTable.getString("AvatarPropertiesUpdate");
char const* const _PREHASH_ParcelPropertiesUpdate = gMessageStringTable.getString("ParcelPropertiesUpdate");
char const* const _PREHASH_FetchItems = gMessageStringTable.getString("FetchItems");
char const* const _PREHASH_AbortXfer = gMessageStringTable.getString("AbortXfer");
char const* const _PREHASH_DeRezAck = gMessageStringTable.getString("DeRezAck");
char const* const _PREHASH_TakeControls = gMessageStringTable.getString("TakeControls");
char const* const _PREHASH_DirLandReply = gMessageStringTable.getString("DirLandReply");
char const* const _PREHASH_MuteType = gMessageStringTable.getString("MuteType");
char const* const _PREHASH_IMViaEMail = gMessageStringTable.getString("IMViaEMail");
char const* const _PREHASH_RentPrice = gMessageStringTable.getString("RentPrice");
char const* const _PREHASH_GenericMessage = gMessageStringTable.getString("GenericMessage");
char const* const _PREHASH_GenericStreamingMessage = gMessageStringTable.getString("GenericStreamingMessage");
char const* const _PREHASH_ChildAgentAlive = gMessageStringTable.getString("ChildAgentAlive");
char const* const _PREHASH_AssetType = gMessageStringTable.getString("AssetType");
char const* const _PREHASH_SpawnPointBlock = gMessageStringTable.getString("SpawnPointBlock");
char const* const _PREHASH_ObjectMaterial = gMessageStringTable.getString("ObjectMaterial");
char const* const _PREHASH_OwnerName = gMessageStringTable.getString("OwnerName");
char const* const _PREHASH_AvatarNotesReply = gMessageStringTable.getString("AvatarNotesReply");
char const* const _PREHASH_CacheID = gMessageStringTable.getString("CacheID");
char const* const _PREHASH_OwnerMask = gMessageStringTable.getString("OwnerMask");
char const* const _PREHASH_TransferInventoryAck = gMessageStringTable.getString("TransferInventoryAck");
char const* const _PREHASH_RegionDenyAgeUnverified = gMessageStringTable.getString("RegionDenyAgeUnverified");
char const* const _PREHASH_AgeVerificationBlock = gMessageStringTable.getString("AgeVerificationBlock");
char const* const _PREHASH_RegionAllowAccessBlock = gMessageStringTable.getString("RegionAllowAccessBlock");
char const* const _PREHASH_RegionAllowAccessOverride = gMessageStringTable.getString("RegionAllowAccessOverride");
char const* const _PREHASH_ParcelEnvironmentBlock = gMessageStringTable.getString("ParcelEnvironmentBlock");
char const* const _PREHASH_ParcelEnvironmentVersion = gMessageStringTable.getString("ParcelEnvironmentVersion");
char const* const _PREHASH_ParcelExtendedFlags = gMessageStringTable.getString("ParcelExtendedFlags");
char const* const _PREHASH_RegionAllowEnvironmentOverride = gMessageStringTable.getString("RegionAllowEnvironmentOverride");
char const* const _PREHASH_UCoord = gMessageStringTable.getString("UCoord");
char const* const _PREHASH_VCoord = gMessageStringTable.getString("VCoord");
char const* const _PREHASH_FaceIndex = gMessageStringTable.getString("FaceIndex");
char const* const _PREHASH_StatusData = gMessageStringTable.getString("StatusData");
char const* const _PREHASH_ProductName = gMessageStringTable.getString("ProductName");
char const* const _PREHASH_ProductSKU = gMessageStringTable.getString("ProductSKU");
char const* const _PREHASH_ColoName = gMessageStringTable.getString("ColoName");
char const* const _PREHASH_CPUClassID = gMessageStringTable.getString("CPUClassID");
char const* const _PREHASH_CPURatio = gMessageStringTable.getString("CPURatio");
char const* const _PREHASH_Language = gMessageStringTable.getString("Language");
char const* const _PREHASH_SeeAVs = gMessageStringTable.getString("SeeAVs");
char const* const _PREHASH_AnyAVSounds = gMessageStringTable.getString("AnyAVSounds");
char const* const _PREHASH_GroupAVSounds = gMessageStringTable.getString("GroupAVSounds");
char const* const _PREHASH_AppearanceData = gMessageStringTable.getString("AppearanceData");
char const* const _PREHASH_AppearanceVersion = gMessageStringTable.getString("AppearanceVersion");
char const* const _PREHASH_CofVersion = gMessageStringTable.getString("CofVersion");
char const* const _PREHASH_Experience = gMessageStringTable.getString("Experience");
char const* const _PREHASH_ExperienceID = gMessageStringTable.getString("ExperienceID");
char const* const _PREHASH_AppearanceHover = gMessageStringTable.getString("AppearanceHover");
char const* const _PREHASH_HoverHeight = gMessageStringTable.getString("HoverHeight");
char const* const _PREHASH_LargeGenericMessage = gMessageStringTable.getString("LargeGenericMessage");
char const* const _PREHASH_CombatSettings = gMessageStringTable.getString("CombatSettings");
char const* const _PREHASH_CombatFlags = gMessageStringTable.getString("CombatFlags");
char const* const _PREHASH_OnDeath = gMessageStringTable.getString("OnDeath");
char const* const _PREHASH_DamageThrottle = gMessageStringTable.getString("DamageThrottle");
char const* const _PREHASH_RegenerationRate = gMessageStringTable.getString("RegenerationRate");
char const* const _PREHASH_InvulnerabilyTime = gMessageStringTable.getString("InvulnerabilyTime");
char const* const _PREHASH_DamageLimit = gMessageStringTable.getString("DamageLimit");
char const* const _PREHASH_GameControlInput = gMessageStringTable.getString("GameControlInput");
char const* const _PREHASH_AxisData = gMessageStringTable.getString("AxisData");
char const* const _PREHASH_MetaData = gMessageStringTable.getString("MetaData");
#if LL_PUPPETRY
char const* const _PREHASH_AgentAnimationTracking = gMessageStringTable.getString("AgentAnimationTracking");
char const* const _PREHASH_AttachmentPointUpdate = gMessageStringTable.getString("AttachmentPointUpdate");
#endif

// Variable region size support
char const* const _PREHASH_RegionSizeX = gMessageStringTable.getString("RegionSizeX");
char const* const _PREHASH_RegionSizeY = gMessageStringTable.getString("RegionSizeY");
char const* const _PREHASH_SizeX = gMessageStringTable.getString("SizeX");
char const* const _PREHASH_SizeY = gMessageStringTable.getString("SizeY");

char const* const _PREHASH_ObjectCapacity = gMessageStringTable.getString("ObjectCapacity");

#if 0	// Old economy data, never used...
char const* const _PREHASH_ObjectCount = gMessageStringTable.getString("ObjectCount");
char const* const _PREHASH_PriceEnergyUnit = gMessageStringTable.getString("PriceEnergyUnit");
char const* const _PREHASH_PriceObjectClaim = gMessageStringTable.getString("PriceObjectClaim");
char const* const _PREHASH_PricePublicObjectDecay = gMessageStringTable.getString("PricePublicObjectDecay");
char const* const _PREHASH_PricePublicObjectDelete = gMessageStringTable.getString("PricePublicObjectDelete");
char const* const _PREHASH_PriceRentLight = gMessageStringTable.getString("PriceRentLight");
char const* const _PREHASH_TeleportMinPrice = gMessageStringTable.getString("TeleportMinPrice");
#endif
