/**
 * @file llappviewerwindows.h
 * @brief The LLAppViewerWindows class declaration
 *
 * $LicenseInfo:firstyear=2007&license=viewergpl$
 *
 * Copyright (c) 2007-2009, Linden Research, Inc.
 *
 * Second Life Viewer Source Code
 * The source code in this file ("Source Code") is provided by Linden Lab
 * to you under the terms of the GNU General Public License, version 2.0
 * ("GPL"), unless you have obtained a separate licensing agreement
 * ("Other License"), formally executed by you and Linden Lab.  Terms of
 * the GPL can be found in doc/GPL-license.txt in this distribution, or
 * online at http://secondlifegrid.net/programs/open_source/licensing/gplv2
 *
 * There are special exceptions to the terms and conditions of the GPL as
 * it is applied to this Source Code. View the full text of the exception
 * in the file doc/FLOSS-exception.txt in this software distribution, or
 * online at
 * http://secondlifegrid.net/programs/open_source/licensing/flossexception
 *
 * By copying, modifying or distributing this software, you acknowledge
 * that you have read and understood your obligations described above,
 * and agree to abide by those obligations.
 *
 * ALL LINDEN LAB SOURCE CODE IS PROVIDED "AS IS." LINDEN LAB MAKES NO
 * WARRANTIES, EXPRESS, IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY,
 * COMPLETENESS OR PERFORMANCE.
 * $/LicenseInfo$
 */

#ifndef LL_LLAPPVIEWERWINDOWS_H
#define LL_LLAPPVIEWERWINDOWS_H

#include "llappviewer.h"

class LLAppViewerWindows final : public LLAppViewer
{
protected:
	LOG_CLASS(LLAppViewerWindows);

public:
	LLAppViewerWindows(const char* cmd_line);

	InitState init() override;
	bool cleanup() override;

	bool probeVulkan(std::string& version) override;

protected:
	void initLogging() override; // Override to clean stack_trace info.
	void initConsole() override; // Initialize OS level debugging console.
	bool initHardwareTest() override; // Windows uses DX9 to test hardware.
	bool initParseCommandLine(LLCommandLineParser& clp) override;

	bool beingDebugged() override;

	bool restoreErrorTrap() override;
	void handleSyncCrashTrace() override;

	bool sendURLToOtherInstance(const std::string& url) override;

	std::string generateSerialNumber() override;

protected:
	static const std::string	sWindowClass;

private:
	bool						mIsConsoleAllocated;
    std::string					mCmdLine;
};

#endif // LL_LLAPPVIEWERWINDOWS_H
