/**
 * @file lldrawpoolsky.h
 * @brief LLDrawPoolSky class definition
 *
 * $LicenseInfo:firstyear=2002&license=viewergpl$
 *
 * Copyright (c) 2002-2009, Linden Research, Inc.
 *
 * Second Life Viewer Source Code
 * The source code in this file ("Source Code") is provided by Linden Lab
 * to you under the terms of the GNU General Public License, version 2.0
 * ("GPL"), unless you have obtained a separate licensing agreement
 * ("Other License"), formally executed by you and Linden Lab.  Terms of
 * the GPL can be found in doc/GPL-license.txt in this distribution, or
 * online at http://secondlifegrid.net/programs/open_source/licensing/gplv2
 *
 * There are special exceptions to the terms and conditions of the GPL as
 * it is applied to this Source Code. View the full text of the exception
 * in the file doc/FLOSS-exception.txt in this software distribution, or
 * online at
 * http://secondlifegrid.net/programs/open_source/licensing/flossexception
 *
 * By copying, modifying or distributing this software, you acknowledge
 * that you have read and understood your obligations described above,
 * and agree to abide by those obligations.
 *
 * ALL LINDEN LAB SOURCE CODE IS PROVIDED "AS IS." LINDEN LAB MAKES NO
 * WARRANTIES, EXPRESS, IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY,
 * COMPLETENESS OR PERFORMANCE.
 * $/LicenseInfo$
 */

#ifndef LL_LLDRAWPOOLSKY_H
#define LL_LLDRAWPOOLSKY_H

#include "lldrawpool.h"

class LLSkyTex;

class LLDrawPoolSky final : public LLFacePool
{
public:
	enum
	{
		VERTEX_DATA_MASK =	LLVertexBuffer::MAP_VERTEX |
							LLVertexBuffer::MAP_TEXCOORD0
	};

	LL_INLINE U32 getVertexDataMask() override
	{
		return VERTEX_DATA_MASK;
	}

	LLDrawPoolSky();

	void prerender() override;

	// All the render methods are no-ops with the PBR renderer.

	void render(S32 pass = 0) override;
	LL_INLINE void endRenderPass(S32) override			{}

	// This will return 1 in EE mode and 0 in PBR mode since it indirectly
	// calls LLDrawpool::getNumPasses().
	LL_INLINE S32 getNumPostDeferredPasses() override	{ return getNumPasses(); }
	LL_INLINE void endPostDeferredPass(S32 p) override	{ endRenderPass(p); }
	LL_INLINE void renderPostDeferred(S32 p) override	{ render(p); }

	LL_INLINE void setSkyTex(LLSkyTex* st)				{ mSkyTex = st; }

private:
	void renderSkyFace(U8 side);

private:
	LLSkyTex* mSkyTex;
};

#endif // LL_LLDRAWPOOLSKY_H
